@@ -26,6 +26,12 @@ int comm_get_preferences(struct arglist *);
 int comm_get_rules(struct arglist *);
 
 int comm_send_preferences(struct arglist *);
+int comm_send_long_preferences(struct arglist*);
 int comm_send_rules(struct arglist *);
 void comm_get_preferences_errors(struct arglist *);
+
+harglst * comm_get_sessions();
+int comm_server_restores_sessions(struct arglist *);
+void comm_delete_session(char*);
+void comm_restore_session(char*);
 #endif
