@@ -21,6 +21,7 @@
 #include "report.h"
 #include "report_utils.h"
 #include "globals.h"
+#include "text_output.h"
 #define CATEGORY_A 0
 #define CATEGORY_B 1
 #define CATEGORY_C 2
@@ -28,50 +29,50 @@
 #define CATEGORY_E 4
 
 
-static char * latex_remove_extra_cr(char *);
 static void latex_print_header(FILE *);
 static void latex_print_footer(FILE *);
 
+/* Pluto 25.6.00:
+ * easy rules for a complicated issue ...
+ * What if we have a large network with lots of small holes, is this saver
+ * than a network with only one big? I've made a try on weighted
+ * rules. Hosts with holes get elevated to *100, warnings to *10 and
+ * infos stay where they are. 
+ */
 
-
-
-
-static int latex_report_category(hosts, holes, warnings)
- int hosts, holes, warnings;
+/*
+static int latex_report_category(hosts, holes, warnings, infos)
+ int hosts, holes, warnings, infos;
 {
  if(((holes*100)/hosts) > 10)return CATEGORY_E;
  if(holes)return CATEGORY_D;
  if(((warnings*100)/hosts) > 10)return CATEGORY_C;
  if(warnings)return CATEGORY_B;
  return CATEGORY_A;
-}
+}*/
 
-
-static char * latex_remove_extra_cr(str)
- char * str;
+static int latex_report_category(hosts, holes, warnings, notes)
+ int hosts, holes, warnings, notes;
 {
- int num = 0;
- char * ret;
- int j = 0;
- char * end;
- char * fr = strdup(str);
- 
- str = fr;
- 
- /*
-  * Compute the size we'll need
-  */
-  while(str[0]=='\n')str+=sizeof(char);
-  end = &(str[strlen(str)-1]);
-  while(end[0]=='\n')
-  {
-   str[strlen(str)-1]='\0';
-   end = &(str[strlen(str)-1]);
-  }
-  ret = strdup(str);
-  free(fr);
-  return ret;
+  int pholes, pwarnings, pnotes, all = 0;
+
+  pholes = (int) (holes*100)/hosts;
+  pwarnings = (int) (warnings*100)/hosts;
+  pnotes = (int) (notes*100)/hosts;
+   
+  all = (pholes * 10) + (pwarnings * 5) + pnotes;
+
+#ifdef DEBUG
+  fprintf(stderr,"%s:%d pholes: %d\tpwarnings: %d\tpnotes: %d\tall: %d\n",__FILE__,__LINE__,pholes, pwarnings, pnotes,all);
+#endif
+
+  if (all > 1200) return CATEGORY_E;
+  else if (all > 600) return CATEGORY_D;
+  else if (all > 300) return CATEGORY_C;
+  else if (all > 0) return CATEGORY_B;
+  else return CATEGORY_A;
 }
+
    
 /*
  * Print the LaTeX header
@@ -93,7 +94,7 @@ static void latex_print_header(FILE * f)
   */
  fprintf(f, "\\pagenumbering{roman}\n");
  fprintf(f, "\\title{\\vspace*{100pt}\\Huge Report of a Nessus scan\\normalsize}\n");
- fprintf(f, "%\n% You might want to change this : \n%\n");
+ fprintf(f, "%%\n%% You might want to change this : \n%%\n");
  fprintf(f, "\\author{Nessus Security Scanner}\n");
  fprintf(f, "\\begin{document}\n");
  fprintf(f, "\\maketitle\n");
@@ -125,7 +126,7 @@ static void latex_print_footer(FILE *  f)
 /*
  * Nothing found : excellent
  */
-void latex_introduction_a(FILE * f, struct arglist * hosts, int holes, int warnings)
+void latex_introduction_a(FILE * f, struct arglist * hosts, int holes, int warnings, int notes)
 {
  int num_of_hosts = arglist_length(hosts);
  fprintf(f, "\\newpage\n");
@@ -151,7 +152,7 @@ void latex_introduction_a(FILE * f, struct arglist * hosts, int holes, int warni
 /*
  * Some things found, but not serious on the overall
  */
-void latex_introduction_b(FILE * f, struct arglist * hosts, int holes, int warnings)
+void latex_introduction_b(FILE * f, struct arglist * hosts, int holes, int warnings, int notes)
 {
  struct arglist * most = most_dangerous_host(hosts);
  int num_of_hosts = arglist_length(hosts);
@@ -159,8 +160,8 @@ void latex_introduction_b(FILE * f, struct arglist * hosts, int holes, int warni
  fprintf(f, "\\section*{Introduction}\n");
  fprintf(f, "In this test, Nessus has tested %d host", num_of_hosts);
  if(num_of_hosts > 1)fprintf(f, "s");
- fprintf(f, " and found %d security warnings that can eventually help a cracker to break ", warnings);
- fprintf(f, "into your network. You should have a close look at them and ");
+ fprintf(f, " and found %d security holes, %d warnings and %d notes that can eventually help a cracker ", holes, warnings, notes);
+ fprintf(f, "to break into your network. You should have a close look at them and ");
  fprintf(f, "decide of their severity. We strongly suggest that you correct them ");
  fprintf(f, "although we know it is not always possible.\\\\\n");
  if((num_of_hosts > 1)&&most)
@@ -169,7 +170,6 @@ void latex_introduction_b(FILE * f, struct arglist * hosts, int holes, int warni
   		most->name);
   fprintf(f, "the most affected by these warnings.\\\\\n");
  }
- fprintf(f, "In spite of these warnings, no major security holes could be found ");
  if(Scanners && Plugins)
  {
  fprintf(f, "(see Appendix A and B page \\pageref{appendix_a} and page \\pageref{appendix_b} for the exhaustive ");
@@ -184,7 +184,7 @@ void latex_introduction_b(FILE * f, struct arglist * hosts, int holes, int warni
 /*
  * A lot of small things have been found. That's not good
  */
-int latex_introduction_c(FILE * f, struct arglist * hosts, int holes, int warnings)
+int latex_introduction_c(FILE * f, struct arglist * hosts, int holes, int warnings, int notes)
 {
  struct arglist * most = most_dangerous_host(hosts);
  int num_of_hosts = arglist_length(hosts);
@@ -192,8 +192,8 @@ int latex_introduction_c(FILE * f, struct arglist * hosts, int holes, int warnin
  fprintf(f, "\\section*{Introduction}\n");
  fprintf(f, "In this test, Nessus has tested %d host", num_of_hosts);
  if(num_of_hosts > 1)fprintf(f, "s");
- fprintf(f, " and found %d security warnings that can eventually help a cracker to break ", warnings);
- fprintf(f, "into your network. You should have a close look at them and ");
+ fprintf(f, " and found %d security holes, %d warnings an %d notes that can eventually help a cracker ", holes, warnings, notes);
+ fprintf(f, "to break into your network. You should have a close look at them and ");
  fprintf(f, "decide of their severity. \\\\\n");
  fprintf(f, "Note that there is a big number of warnings for a single network of this size. ");
  fprintf(f, "What you must know is that what usually compromise the security of a network ");
@@ -207,7 +207,7 @@ int latex_introduction_c(FILE * f, struct arglist * hosts, int holes, int warnin
   		most->name);
   fprintf(f, "the most affected by these warnings.\\\\\n");
  }
- fprintf(f, "In spite of these warnings, no major security holes could be found ");
+
  if(Scanners && Plugins)
  {
  fprintf(f, "(see Appendix A and B page \\pageref{appendix_a} and page \\pageref{appendix_b} for the exhaustive ");
@@ -220,7 +220,7 @@ int latex_introduction_c(FILE * f, struct arglist * hosts, int holes, int warnin
 }
 
 
-int latex_introduction_d(FILE * f, struct arglist * hosts, int holes, int warnings)
+int latex_introduction_d(FILE * f, struct arglist * hosts, int holes, int warnings, int notes)
 {
  struct arglist * most = most_dangerous_host(hosts);
  int num_of_hosts = arglist_length(hosts);
@@ -228,8 +228,7 @@ int latex_introduction_d(FILE * f, struct arglist * hosts, int holes, int warnin
  fprintf(f, "\\section*{Introduction}\n");
  fprintf(f, "In this test, Nessus has tested %d host", num_of_hosts);
  if(num_of_hosts > 1)fprintf(f, "s");
- fprintf(f, " and found \\textbf{%d severe security holes}, as well as %d security warnings.", holes,
-		 warnings);
+ fprintf(f, " and found \\textbf{%d severe security holes}, as well as %d security warnings and %d notes.", holes, warnings, notes);
  fprintf(f, "These problems can easily be used to break ");
  fprintf(f, "into your network. You should have a close look at them and ");
  fprintf(f, "correct them as soon as possible.\\\\\n");
@@ -256,7 +255,7 @@ int latex_introduction_d(FILE * f, struct arglist * hosts, int holes, int warnin
  return 0 ;
 }
 
-int latex_introduction_e(FILE * f, struct arglist * hosts, int holes, int warnings)
+int latex_introduction_e(FILE * f, struct arglist * hosts, int holes, int warnings, int notes)
 {
  struct arglist * most = most_dangerous_host(hosts);
  int num_of_hosts = arglist_length(hosts);
@@ -264,8 +263,7 @@ int latex_introduction_e(FILE * f, struct arglist * hosts, int holes, int warnin
   fprintf(f, "\\section*{Introduction}\n");
  fprintf(f, "In this test, Nessus has tested %d host", num_of_hosts);
  if(num_of_hosts > 1)fprintf(f, "s");
- fprintf(f, " and found \\textbf{%d severe security holes}, as well as %d security warnings.", holes,
-		 warnings);
+ fprintf(f, " and found \\textbf{%d severe security holes}, as well as %d security warnings and %d notes.", holes, warnings, notes);
  fprintf(f, "These problems can easily be used to break ");
  fprintf(f, "into your network. You should have a close look at them and ");
  fprintf(f, "correct them as soon as possible.\\\\\n");
@@ -322,7 +320,8 @@ void latex_print_host_ports_problems(FILE * f, struct arglist * port)
  {
  struct arglist * holes = arg_get_value(port->value, "REPORT");
  struct arglist * info = arg_get_value(port->value, "INFO");
- if(!(holes || info)){
+ struct arglist * note = arg_get_value(port->value, "NOTE");
+ if(!(holes || info || note)){
  	latex_print_host_ports_problems(f, port->next);
 	return;
 	}
@@ -334,9 +333,11 @@ void latex_print_host_ports_problems(FILE * f, struct arglist * port)
     fprintf(f, "\\begin{itemize}\n");
     while(holes->next)
     {
-    char * p = latex_remove_extra_cr(holes->value);
-    fprintf(f, "\\item \\begin{verbatim}\n%s\n\\end{verbatim}", p);
-    free(p);
+    char * c = holes->value;
+    while(c[0]=='\n')c++;
+    fprintf(f, "\\item \\begin{verbatim}\n");
+    printf_formatted_text(f, c, 70, NULL);
+    fprintf(f, "\n\\end{verbatim}");
     holes = holes->next;
     }
     fprintf(f, "\\end{itemize}\n");
@@ -347,13 +348,30 @@ void latex_print_host_ports_problems(FILE * f, struct arglist * port)
     fprintf(f, "\\begin{itemize}\n");
     while(info->next)
     {
-     char * p = latex_remove_extra_cr(info->value);
-     fprintf(f, "\\item \\begin{verbatim}\n%s\n\\end{verbatim}", p);
-     free(p);
+     char * c = info->value;
+     while(c[0]=='\n')c++;
+     fprintf(f, "\\item \\begin{verbatim}\n");
+     printf_formatted_text(f, c, 70, NULL);
+     fprintf(f, "\n\\end{verbatim}");
      info = info->next;
     }
     fprintf(f, "\\end{itemize}\n");
    }
+   if(note)
+   {
+    fprintf(f, "Security note :\\\\\n");
+    fprintf(f, "\\begin{itemize}\n");
+    while(note->next)
+    {
+     char * c = note->value;
+     while(c[0]=='\n')c++;
+     fprintf(f, "\\item \\begin{verbatim}\n");
+     printf_formatted_text(f, c, 70, NULL);
+     fprintf(f, "\n\\end{verbatim}");
+     note = note->next;
+    }
+    fprintf(f, "\\end{itemize}\n");
+   }
   }
   latex_print_host_ports_problems(f, port->next);
  }
@@ -387,6 +405,7 @@ void latex_print_hosts(FILE * f, struct arglist * host)
    * Detail of the problems
    */
   if(number_of_holes_by_port(arg_get_value(host->value, "PORTS")) ||
+     number_of_notes_by_port(arg_get_value(host->value, "PORTS")) ||
      number_of_warnings_by_port(arg_get_value(host->value, "PORTS")))
    {
      fprintf(f, "\\subsection{Details of the vulnerabilities}\n");
@@ -448,7 +467,7 @@ void latex_appendix_plugins(FILE * f, struct arglist * plugins)
    if(plug_get_launch(plugins->value))
    {
     fprintf(f, "\\item \\verb+%s+\n", 
-    			arg_get_value(plugins->value, "NAME"));
+    			(char*)arg_get_value(plugins->value, "NAME"));
    }
    latex_appendix_plugins(f, plugins->next);
   }
@@ -474,11 +493,14 @@ void latex_appendix(FILE *f)
  				
 int arglist_to_latex(struct arglist * list, char * filename)
 {
- FILE * f = fopen(filename, "w");
+ FILE * f;
+ int notes = number_of_notes(list);
  int warnings = number_of_warnings(list);
  int holes = number_of_holes(list);
  int length = arglist_length(list);
  
+ if(!strcmp(filename, "-"))f = stdout;
+ else f = fopen(filename, "w");
  if(!f){
  	perror("open ");
 	return -1;
@@ -498,18 +520,18 @@ int arglist_to_latex(struct arglist * list, char * filename)
   *	- more than 10% of warnings : network security must be improved _C
   */
  
-  switch(latex_report_category(length, holes, warnings))
+  switch(latex_report_category(length, holes, warnings, notes))
   {
    case CATEGORY_A :
-   	 latex_introduction_a(f, list, holes, warnings);break;
+   	 latex_introduction_a(f, list, holes, warnings, notes);break;
    case CATEGORY_B :
-  	 latex_introduction_b(f, list, holes, warnings);break;
+  	 latex_introduction_b(f, list, holes, warnings, notes);break;
    case CATEGORY_C :
-  	 latex_introduction_c(f, list, holes, warnings);break;
+  	 latex_introduction_c(f, list, holes, warnings, notes);break;
    case CATEGORY_D :
-  	 latex_introduction_d(f, list, holes, warnings);break;
+  	 latex_introduction_d(f, list, holes, warnings, notes);break;
    default :
-   	 latex_introduction_e(f, list, holes, warnings);break;
+   	 latex_introduction_e(f, list, holes, warnings, notes);break;
    
   }
   
