@@ -1,5 +1,5 @@
 /* Nessus
- * Copyright (C) 1998 Renaud Deraison
+ * Copyright (C) 1998, 1999, 2000 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,17 +31,19 @@
 #include "report.h"
 #include "globals.h"
 #include "error_dialog.h"
+#include "xpm/computer.xpm"
 
-static char ** plugins_order_table = NULL;
-static int plugins_in_table = 0;
+/*static char ** plugins_order_table = NULL;*/
+static harglst * plugins_order_table = NULL;
 
 static void monitor_input_callback(struct arglist *, int, int);
 static void monitor_add_host(struct arglist *, char *, int);
 static void monitor_stop_test(GtkWidget *, char *);
-static void monitor_list_update(struct arglist *, char *);
+static void monitor_list_update(struct arglist *, char *, int);
+static int monitor_stop_whole_test_destroy(void*, void*, struct arglist *);
 static int monitor_stop_whole_test(GtkWidget * , struct arglist *);
 static void build_plugins_order_table(char *);
-
+static char * label_name(char * , GtkWidget * , GtkWidget * );
 
 /*
  * Function called when the UI is idle, which checks
@@ -54,12 +56,12 @@ idle_socket(struct arglist * ctrls)
 {
   fd_set rd;
   struct timeval tv = {0,100};
-
+  int n;
   if(GlobalSocket < 0)return FALSE;
   FD_ZERO(&rd);
   FD_SET(GlobalSocket, &rd);
-  select(GlobalSocket+1, &rd, NULL, NULL, &tv);
-  if(FD_ISSET(GlobalSocket, &rd))
+  n = select(GlobalSocket+1, &rd, NULL, NULL, &tv);
+  if(FD_ISSET(GlobalSocket, &rd)&& (n > 0))
     monitor_input_callback(ctrls, GlobalSocket, 0);
   return TRUE;
 }
@@ -71,20 +73,25 @@ idle_socket(struct arglist * ctrls)
  * show the attack status
  */
 void 
-monitor_dialog_setup(victim)
+monitor_dialog_setup(victim, restore)
      char * victim;
+     int restore;
 {
   struct arglist * ctrls = emalloc(sizeof(struct arglist));
   GtkWidget * scrolled_window; 
   GtkWidget * w, * box;
   int tag;
-  MonitorDialog = ctrls;
-  Hosts = emalloc(sizeof(struct arglist));
-  arg_add_value(ctrls, "HOSTS", ARG_ARGLIST, -1, Hosts);
+  struct arglist * hosts;
+
+  hosts = emalloc(sizeof(struct arglist));
+
+  arg_add_value(ctrls, "HOSTS", ARG_ARGLIST, -1, hosts);
   
   w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_default_size(GTK_WINDOW(w), 640,480);
+  gtk_widget_realize(w);
   gtk_signal_connect(GTK_OBJECT(w), "delete_event",
-		     (GtkSignalFunc)monitor_stop_whole_test,ctrls);
+		     (GtkSignalFunc)monitor_stop_whole_test_destroy,ctrls);
   gtk_window_set_title(GTK_WINDOW(w), "Nessus portscanning/attack status");
   gtk_container_border_width(GTK_CONTAINER(w), 10);
   arg_add_value(ctrls, "WINDOW", ARG_PTR, -1, w);
@@ -99,8 +106,7 @@ monitor_dialog_setup(victim)
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 				
-  gtk_container_border_width(GTK_CONTAINER(scrolled_window), 10);   
-  gtk_widget_set_usize(scrolled_window, 500,300); 
+  gtk_container_border_width(GTK_CONTAINER(scrolled_window), 10);  
   gtk_box_pack_start(GTK_BOX(box), scrolled_window, TRUE, TRUE, 0);
   
   w = gtk_list_new();
@@ -113,19 +119,22 @@ monitor_dialog_setup(victim)
   gtk_widget_show (scrolled_window);
   arg_add_value(ctrls, "LIST", ARG_PTR, -1, w);
                     
-#if GTK_VERSION < 12
-  gtk_idle_add(idle_socket, ctrls);
-#else
-  g_idle_add(idle_socket, ctrls);
-#endif
+
   
+  tag = gtk_idle_add((GtkFunction)idle_socket, ctrls);
+  
+  arg_add_value(ctrls, "TAG", ARG_INT, sizeof(int), (void*)tag);
   w = gtk_button_new_with_label("Stop the whole test");
   gtk_signal_connect(GTK_OBJECT(w), "clicked",
 		     (GtkSignalFunc)monitor_stop_whole_test,ctrls);
   gtk_box_pack_start(GTK_BOX(box), w, FALSE, TRUE, 0);
   gtk_widget_show(w);
-  
-  attack_host(victim, Prefs);
+#if ENABLE_SAVE_TESTS
+  if(restore)
+   restore_attack(victim, Prefs);
+  else
+#endif
+   attack_host(victim, Prefs);
 }
 
 /*
@@ -134,13 +143,14 @@ monitor_dialog_setup(victim)
  * Updates the progress bars
  */	
 static void 
-monitor_list_update(ctrls,msg)
+monitor_list_update(ctrls,msg, short_status)
 	struct arglist * ctrls;
 	char * msg;
+	int short_status;
 {
  char * hostname;
  char * action;
- int current;
+ char* current = NULL;
  int max;
  gfloat gmax;
  gfloat gcurrent;
@@ -150,10 +160,38 @@ monitor_list_update(ctrls,msg)
  char * list_hostname;
  int flag = 0;
  
- 
- hostname = emalloc(strlen(msg)+1);
- action = emalloc(strlen(msg)+1);
- parse_nessusd_status(msg, hostname, action, &current, &max);
+ if(!short_status)
+ {
+  hostname = emalloc(strlen(msg)+1);
+  action = emalloc(strlen(msg)+1);
+  parse_nessusd_status(msg, hostname, action, &current, &max);
+ }
+ else
+ {
+  char * t;
+  /* the short status is : action:hostname:current:max */
+  if(msg[0]=='p')
+   action = "portscan";
+  else
+   action  = "attack";
+  
+  msg = msg + sizeof(char)*2; 
+  t = strchr(msg, ':');
+  if(!t)
+   return; 
+  t[0] = 0;
+  hostname = strdup(msg);
+  
+  msg = t + sizeof(char);
+  t = strchr(msg, ':');
+  
+  if(!t)
+   return;
+  t[0]=0;
+  current = strdup(msg);
+  msg = t + sizeof(char);
+  max = atoi(msg);
+ }
  gtkw = arg_get_value(ctrls, "LIST");
  dlist = GTK_LIST(gtkw)->children;
  while(dlist && !flag)
@@ -170,6 +208,8 @@ monitor_list_update(ctrls,msg)
    GtkWidget * progress_bar;
    gfloat f;
    GtkWidget * label; 
+   char * lbl;
+   
    if(!strcmp(action, "portscan"))
     progress_bar = gtk_object_get_data(item, "progress_bar_portscan");
    else
@@ -177,13 +217,23 @@ monitor_list_update(ctrls,msg)
     progress_bar = gtk_object_get_data(item, "progress_bar_attack");
     label = gtk_object_get_data(item, "current_plugin");
    
-    
-    if((current> 0) && (current <= plugins_in_table))
-     gtk_label_set(GTK_LABEL(label), plugins_order_table[current-1]);
-    else gtk_label_set(GTK_LABEL(label), "???");
+   
+    if(current)
+        {
+	 char * str = harg_get_string(plugins_order_table, current);
+	 if(str)
+    	  lbl = label_name(str, progress_bar, arg_get_value(ctrls, "WINDOW"));
+	 else
+	  lbl = label_name("???", progress_bar, arg_get_value(ctrls, "WINDOW"));
+	}
+    else
+        lbl = label_name("???", progress_bar, arg_get_value(ctrls, "WINDOW"));
+	
+    gtk_label_set(GTK_LABEL(label), lbl);
+    free(lbl);
     }
    gmax = max;
-   gcurrent = current;
+   gcurrent = atoi(current);
    f = (gcurrent/gmax);
    if(f>1)f=1;
    if(f<0.0)f=0.0;
@@ -197,8 +247,10 @@ monitor_list_update(ctrls,msg)
  if(!flag)
  {
  /* the host was not found, we must add one... */
- monitor_add_host(ctrls, hostname, current);
+ monitor_add_host(ctrls, hostname, atoi(current));
  }
+
+ if(current)free(current);
  
 }
  
@@ -206,29 +258,73 @@ monitor_list_update(ctrls,msg)
  * monitor_remove_host
  */
 void
-monitor_remove_host(host)
+monitor_remove_host(ctrls, host)
+ struct arglist * ctrls;
  char * host;
 {
- struct arglist * ctrls = MonitorDialog;
  GtkWidget * item;
+ GList * list = NULL;
  item =  gtk_object_get_data(GTK_OBJECT(arg_get_value(ctrls, "LIST")),host);
+ 
  if(!item){
+#ifndef ENABLE_SAVE_KB
 	/*
 	 * If this happens, then it's very likely that the server
 	 * thinks the communication has been cut between the client
 	 * and itself. Which is not a good thing.
 	 */
- 	printf("warning. Could not find entry for %s\n", host);
-	printf("This may be a bug - please check the nessusd logfile\n");
-	printf("and if you see something about a lost connection,\n");
-	printf("or any odd message, please report it to deraison@cvs.nessus.org\n");
+ 	fprintf(stderr, "warning. Could not find entry for %s\n", host);
+	fprintf(stderr, "This may be a bug - please check the nessusd logfile\n");
+	fprintf(stderr, "and if you see something about a lost connection,\n");
+	fprintf(stderr, "or any odd message, please report it to deraison@cvs.nessus.org\n");
+#endif	
 	return;
 	}
- gtk_widget_hide(item);	
- /*gtk_list_remove_items(GTK_LIST(arg_get_value(ctrls, "LIST")), dlist);*/
+ 
+ if(item != (void*)-1)
+ {
+ list = g_list_append(list, item);
+ gtk_list_remove_items(GTK_LIST(arg_get_value(ctrls, "LIST")), list);
+ gtk_object_set_data(GTK_OBJECT(arg_get_value(ctrls, "LIST")),host,(void*)-1);
+ g_list_free(list);
+ }
 }
 
 
+
+static char * label_name(char * label, GtkWidget * bar, GtkWidget * window)
+{
+ char * ret = emalloc(strlen(label)+1);
+ GtkStyle * style;
+ int max_len = 195; /* XXX dirty hack : we hardcoded the width of the
+ 		       progress bar */
+ 
+ 
+ style = gtk_widget_get_style (window);
+ if(!label)label = "???";
+ 
+ if(gdk_string_width (style->font, label) >  max_len)
+ {
+  strncpy(ret, label, strlen(label));
+  while(gdk_string_width (style->font, ret) >  max_len)
+  {
+   if(!strlen(ret))break;
+   ret[strlen(ret)-1]='\0';
+  }
+  if(strlen(ret)>3)
+  {
+  ret[strlen(ret)-1]='.';
+  ret[strlen(ret)-2]='.';
+  ret[strlen(ret)-3]='.';
+  }
+ }
+ else
+ {
+  strncpy(ret, label, strlen(label));
+  }
+ return ret;
+}
+ 
 /*
  * monitor_add_host
  *
@@ -250,8 +346,13 @@ monitor_add_host(ctrls,hostname,port)
  GtkWidget * separator;
  GtkWidget * box, * hbox;
  GtkWidget * other_label;
+ GtkStyle  * style;
+ GtkWidget * pixmapwid;
+ GdkPixmap * pixmap = NULL;
+ GdkBitmap * mask;
  GList * dlist;
-
+ GtkWidget * window = arg_get_value(ctrls,"WINDOW");
+ char * name;
  item = gtk_list_item_new();
  dlist = NULL;
  table = gtk_table_new(4, 3, FALSE);
@@ -269,11 +370,24 @@ monitor_add_host(ctrls,hostname,port)
  /*
   * Host name
   */
+
+    
  box = gtk_vbox_new(TRUE,0);
  gtk_widget_show(box);
  gtk_box_pack_start(GTK_BOX(hbox), box, FALSE, FALSE, 0);
+ if(F_show_pixmaps)
+ {
+  style = gtk_widget_get_style(window);
+ pixmap = gdk_pixmap_create_from_xpm_d(window->window, &mask,
+					&style->bg[GTK_STATE_NORMAL],
+					(char **)computer_xpm);
+ pixmapwid = gtk_pixmap_new(pixmap, mask); 
  
- label = gtk_label_new(hostname);
+ gtk_box_pack_start(GTK_BOX(box), pixmapwid, FALSE, FALSE, 0);
+ gtk_widget_show(pixmapwid);
+ }
+ name = hostname;
+ label = gtk_label_new(name);
  gtk_widget_set_usize(label, 150, 15);
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
  gtk_widget_show(label);
@@ -308,21 +422,21 @@ monitor_add_host(ctrls,hostname,port)
  /*
  gtk_table_attach_defaults(GTK_TABLE(table),progress_bar_portscan, 1,2,0,1);
  */
- gtk_box_pack_start(GTK_BOX(box), progress_bar_portscan, FALSE, TRUE,0);
+ gtk_box_pack_start(GTK_BOX(box), progress_bar_portscan, TRUE, TRUE,0);
  gtk_widget_show(progress_bar_portscan);
  
  progress_bar_attack = gtk_progress_bar_new();
  /*
  gtk_table_attach_defaults(GTK_TABLE(table),progress_bar_attack, 1,2,1,2);
  */
- gtk_box_pack_start(GTK_BOX(box), progress_bar_attack, FALSE, TRUE,0);
+ gtk_box_pack_start(GTK_BOX(box), progress_bar_attack, TRUE, TRUE,0);
  gtk_widget_show(progress_bar_attack);
  
  other_label = gtk_label_new("");
  /*
  gtk_table_attach_defaults(GTK_TABLE(table), other_label, 1,2,2,3);
  */
- gtk_box_pack_start(GTK_BOX(box), other_label, FALSE, TRUE, 0);
+ gtk_box_pack_start(GTK_BOX(box), other_label, TRUE, TRUE, 0);
  gtk_widget_show(other_label);
  
  
@@ -352,9 +466,10 @@ monitor_add_host(ctrls,hostname,port)
  gtk_object_set_data(GTK_OBJECT(item), "current_plugin", other_label);
  gtk_widget_show(table);
  gtk_widget_show(item);
+ if(pixmap)
+  gtk_object_set_data(GTK_OBJECT(item), "pixmap", pixmap);
  dlist = g_list_append(dlist, item);
- gtk_object_set_data(GTK_OBJECT(arg_get_value(ctrls,
- "LIST")),estrdup(hostname),item);
+ gtk_object_set_data(GTK_OBJECT(arg_get_value(ctrls, "LIST")),estrdup(hostname),item);
  gtk_list_append_items(GTK_LIST(arg_get_value(ctrls, "LIST")), dlist);
  }
  
@@ -367,23 +482,24 @@ monitor_add_host(ctrls,hostname,port)
  * to the screen
  */
  
+static int
+monitor_stop_whole_test_destroy(a,b,ctrls)
+ void * a, * b;
+ struct arglist * ctrls;
+{
+ return monitor_stop_whole_test(NULL, ctrls);
+}
 static int
 monitor_stop_whole_test(w, ctrls)
  GtkWidget * w;
  struct arglist * ctrls;
 {
-struct arglist * a = arg_get_value(MainDialog, "AUTH");
-        	
- gtk_widget_hide(arg_get_value(a, "CONNECTED"));
- gtk_widget_show(arg_get_value(a, "BUTTON"));
  network_printf( "CLIENT <|> STOP_WHOLE_TEST <|> CLIENT\n");
- shutdown(GlobalSocket, 2);
- closesocket(GlobalSocket);
- GlobalSocket = -1;
  efree(&ArgSock);
  F_nessusd_running = 0;
  gtk_widget_hide(arg_get_value(ctrls, "WINDOW"));
- report_tests(arg_get_value(ctrls, "HOSTS"));
+ gtk_idle_remove((int)arg_get_value(ctrls, "TAG"));
+ report_tests(arg_get_value(ctrls, "HOSTS"), 0);
  return(FALSE);
 }
 
@@ -404,7 +520,7 @@ monitor_input_callback(ctrls, fd, condition)
   char buf [4096];
   char * msg;
   int n, type = -1;
-  
+  int interrupted = 0;
   network_gets_raw(buf, 4095);
   /* if(strlen(buf))buf[strlen(buf)-1]=0; */
   if ((n = strlen (buf)) && buf [n-1] == '\n') buf [n-1] = '\0';
@@ -415,7 +531,11 @@ monitor_input_callback(ctrls, fd, condition)
   if (buf [0] == '\0') {
     n = (errno == 0) ? io_ctrl (GlobalSocket, IO_EOF_STATE, 0, 0) : -1 ;
     if (n == 0) return ;
-    if (n  > 0) goto scan_finished;
+    if (n  > 0) {
+    	/*show_error("Error : nessusd shut down the connection during the test !");*/
+    	interrupted++;
+	goto scan_finished;
+	}
     sprintf (buf, "COMMUNICATION ERROR %u: %s", errno, peks_strerr (errno));
     show_error (buf) ;
     goto scan_finished;
@@ -423,43 +543,41 @@ monitor_input_callback(ctrls, fd, condition)
 #endif
   msg = emalloc(strlen(buf)+1);
   if(strlen(buf))type=parse_server_message(buf, arg_get_value(ctrls, "HOSTS"), msg);
-   else type = MSG_BYE;
+   else {
+   	/*show_error("Error : nessusd shut down the connection during the test !");*/
+	interrupted++;
+   	goto scan_finished;
+	}
   switch(type)
   {
   	case MSG_BYE : 
-        	{
-                struct arglist * a = arg_get_value(MainDialog, "AUTH");
-        	GlobalSocket = -1;
-                gtk_widget_hide(arg_get_value(a, "CONNECTED"));
-                gtk_widget_show(arg_get_value(a, "BUTTON"));
 		network_printf("CLIENT <|> BYE <|> ACK\n");
   		finished = 1;
-                }
   		break;
+	case MSG_STAT2 : 
+		monitor_list_update(ctrls,buf+2, 1);	
+		break;
   	case MSG_STAT :
-  	 	monitor_list_update(ctrls, msg);
+  	 	monitor_list_update(ctrls, msg, 0);
   		break;
         case MSG_PLUGINS_ORDER :
         	build_plugins_order_table(msg);
                 break;
+	case MSG_FINISHED :
+		monitor_remove_host(ctrls, msg);
+		break;	
   }
   bzero(buf, 1024);
   efree(&msg);
   
   if(finished)
     {
-#ifdef ENABLE_CRYPTO_LAYER
    scan_finished : 
-#endif 
-#ifdef DEBUG
-      fprintf(stderr, "Scan finished\n");
-#endif
       F_nessusd_running = 0;
-      shutdown(GlobalSocket,2);
-      closesocket(GlobalSocket);
-      GlobalSocket = -1;
       gtk_widget_hide(arg_get_value(ctrls, "WINDOW"));
-      report_tests(arg_get_value(ctrls, "HOSTS"));
+      gtk_idle_remove((int)arg_get_value(ctrls, "TAG"));
+      gtk_widget_destroy(arg_get_value(ctrls, "WINDOW"));
+      report_tests(arg_get_value(ctrls, "HOSTS"), interrupted);
     }
 }
  
@@ -505,7 +623,8 @@ static void build_plugins_order_table(order)
  /*
   * Now, create the names table
   */
-  plugins_order_table = emalloc(num*sizeof(char *));
+  plugins_order_table = harg_create(num*3 + 1);
+  
   for(i=0;i<num;i++)
   {
    struct arglist * plugs = Plugins;
@@ -520,9 +639,13 @@ static void build_plugins_order_table(order)
         ((int)(arg_get_value(plugs->value, "ID"))!=plugins_order_table_int[i]))
          plugs = plugs->next;
    }
-   if(plugs)plugins_order_table[i] = plugs->name;
+   if(plugs){
+   	char * id = emalloc(20);
+	sprintf(id, "%d", i+1);
+   	harg_add_string(plugins_order_table, id, plugs->name);
+	efree(&id);
+	}
   }
-  plugins_in_table = num;
   efree(&plugins_order_table_int);
 }
 #endif
