@@ -1,5 +1,5 @@
 /* Nessus
- * Copyright (C) 1998 Renaud Deraison
+ * Copyright (C) 1998, 1999, 2000 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "prefs_dialog/prefs_dialog.h"
 #include "prefs_dialog/prefs_dialog_plugins_prefs.h"
 #include "prefs_dialog/prefs_dialog_scan_opt.h"
+#include "prefs_dialog/prefs_target.h"
 #endif
 
 #include "read_target_file.h"
@@ -38,21 +39,39 @@
 #include "report.h"
 #include "parser.h"
 #include "nsr_output.h"
+#include "text_output.h"
+#include "latex_output.h"
+#include "html_output.h"
+#include "html_graph_output.h"
+#include "xml_output.h"
 #include "sighand.h"
 #include "preferences.h"
 #include "globals.h"
 #include "corevers.h"
 #include "nessus/getopt.h"
+#include "splash_screen.h"
+#include "password_dialog.h"
+
+#include "cli.h"
+
+
+
 
 struct arglist * Plugins = NULL;
 struct arglist * Scanners = NULL;
+#ifdef ENABLE_SAVE_TESTS
+harglst * Sessions = NULL;
+int Sessions_saved = 0;
+#endif
+#ifdef ENABLE_SAVE_KB
+int DetachedMode = 0;
+#endif
+
 int PluginsNum;
 int ScannersNum;
-struct arglist * Hosts = NULL;
 struct arglist * Prefs;
 struct arglist * MainDialog;
 struct arglist * ArgSock;
-struct arglist * MonitorDialog;
 char * Alt_rcfile = NULL;
 int GlobalSocket;
 char * stored_pwd = NULL;
@@ -61,6 +80,10 @@ int F_quiet_mode;
 int F_nessusd_running;
 int First_time = 0;
 
+
+
+
+
 #ifndef USE_AF_INET      
 #undef ENABLE_CRYPTO_LAYER
 #endif
@@ -68,12 +91,15 @@ int First_time = 0;
 #ifdef ENABLE_CRYPTO_LAYER
 #include "peks/peks.h"
 #include <errno.h>
-static unsigned keylen = NESSUS_SIGKEYLEN ;
-static char        *kf = NESSUS_KEYFILE ;
-static peks_key   *key = 0;
+typedef char*(*peks_get_pwd_t)(int);
+peks_get_pwd_t peks_get_pwd = get_pwd;
+
+unsigned keylen = NESSUS_SIGKEYLEN ;
+char        *kf = NESSUS_KEYFILE ;
+peks_key   *key = 0;
 
 /* key file processing options ... */
-static int chng_pph = 0, list_kdb = 0, first_login = 0 ;
+static int chng_pph = 0, list_kdb = 0;
 static char *kill_key = 0 ;
 
 #ifdef USE_GTK
@@ -105,25 +131,25 @@ key_initialization_stuff
   int did_something = 0 ;
 
   if (chng_pph) { /* change pass phrase and exit */
-    if ((key = peks_private_key (0, kf, get_pwd, keylen)) == 0) goto error ;
+    if ((key = peks_private_key (0, kf, peks_get_pwd, keylen)) == 0) goto error ;
     ntconsole (10, 80, 0, 0);
     action_header ("Change pass phrase");
     if (!created_private_key () &&
-	peks_save_private_key (0, key, kf, get_pwd) < 0) goto error ;
+	peks_save_private_key (0, key, kf, peks_get_pwd) < 0) goto error ;
     did_something ++ ;
   }
 
   if (kill_key != 0) { /* delete entry */
     if (strchr (kill_key, '@') != 0) { /* syntax is: usr@host */
-      if (peks_delete_userkey (kill_key, kf, get_pwd) < 0) goto error ;
+      if (peks_delete_userkey (kill_key, kf, peks_get_pwd) < 0) goto error ;
     } else
-      if (peks_delete_hostkey (kill_key, kf, get_pwd) < 0) goto error ;
+      if (peks_delete_hostkey (kill_key, kf, peks_get_pwd) < 0) goto error ;
     did_something ++ ;
   }
 
   if (list_kdb) { /* list key data base */
     if (key == 0 &&
-        (key = peks_private_key (0, kf, get_pwd, keylen)) == 0) goto error ;
+        (key = peks_private_key (0, kf, peks_get_pwd, keylen)) == 0) goto error ;
     ntconsole (40, 80, 10, 70);
     action_header ("Key data base");
     if (peks_list_keyfile (0, kf) < 0) goto error ;
@@ -134,7 +160,7 @@ key_initialization_stuff
     exit (0);
 
   /* get your private key */
-  if (key != 0 || (key = peks_private_key (0, kf, get_pwd, keylen)) != 0)
+  if (key != 0 || (key = peks_private_key (0, kf, peks_get_pwd, keylen)) != 0)
     return ;
 
   error:
@@ -175,7 +201,7 @@ connect_to_nessusd(hostname, port, login, pass)
 	char * pass; /* is a cipher in case of the crypto layer */
 {
   int soc;
-  int sz = 4096;
+  
 #ifndef USE_AF_INET
   struct sockaddr_un address;
   char * name = AF_UNIX_PATH;
@@ -186,6 +212,13 @@ connect_to_nessusd(hostname, port, login, pass)
   else
    arg_add_value(Prefs, "nessusd_host", ARG_STRING, strlen(hostname),
    		strdup(hostname));
+			
+  if(arg_get_type(Prefs, "nessusd_user")>=0)
+   arg_set_value(Prefs, "nessusd_user", strlen(login), strdup(login));
+  else
+   arg_add_value(Prefs, "nessusd_user", ARG_STRING, strlen(login),
+   		strdup(login));		
+   
    
 #ifdef USE_AF_INET
   soc = open_sock_tcp_hn(hostname, port);
@@ -243,7 +276,7 @@ connect_to_nessusd(hostname, port, login, pass)
   } else
 
     /* so we are asked to try the insecure way */
-    if(comm_init(soc,"< NTP/1.1 >\n") == 0) {
+    if(comm_init(soc,"< NTP/1.2 >\n") == 0) {
       if (pass == 0 || strcasecmp (pass, "none") == 0) {
 	if (auth_login(login, 0) != 0)
 	  goto wrong_protocol ;
@@ -254,16 +287,18 @@ connect_to_nessusd(hostname, port, login, pass)
 	  goto wrong_protocol ;
     } else
 #else
-      if(comm_init(soc,"< NTP/1.1 >\n") || (auth_login(login, pass)))
+      if(comm_init(soc,"< NTP/1.2 >\n") || (auth_login(login, pass)))
 #endif /* ENABLE_CRYPTO_LAYER */
   	{
+#ifdef ENABLE_CRYPTO_LAYER	
 	wrong_protocol:
+#endif	
         shutdown(soc, 2);
-  	return("Remote host is not using the good version of the Nessus communication protocol (1.1)");
+  	return("Remote host is not using the good version of the Nessus communication protocol (1.2)");
         }
   if(comm_get_pluginlist())return("Login failed");
   if(!First_time){
-  	comm_get_preferences(Prefs);
+	comm_get_preferences(Prefs);
   	comm_get_rules(Prefs);
 	}
   else
@@ -276,6 +311,18 @@ connect_to_nessusd(hostname, port, login, pass)
 	comm_get_rules(devnull);
 	arg_free(devnull);
 	}
+#ifdef ENABLE_SAVE_TESTS
+	if(comm_server_restores_sessions(Prefs))
+	  {
+	  Sessions = comm_get_sessions();
+#ifdef USE_GTK	  
+	  prefs_dialog_target_fill_sessions(arg_get_value(MainDialog, "TARGET"),
+	  		                    Sessions);
+#endif					    
+	 }
+#endif	
+ 
+  	
   prefs_check_defaults(Prefs);
 #ifdef USE_GTK
   prefs_plugins_reset(arg_get_value(MainDialog, "PLUGINS_PREFS"), Plugins,
@@ -305,7 +352,7 @@ connect_to_nessusd(hostname, port, login, pass)
  * init_globals
  *
  * initializes two main global variables : plugins and
- * hosts
+ * scanners
  *
  */
 void 
@@ -313,7 +360,6 @@ init_globals()
 {
   if(!Plugins)Plugins = emalloc(sizeof(struct arglist));
   if(!Scanners)Scanners = emalloc(sizeof(struct arglist));
-  Hosts = emalloc(sizeof(struct arglist));
 }
 
 
@@ -351,6 +397,15 @@ printf ("\
   -L, --list-keys                     list all keys in the data base\n\
   -K KEY, --delete-key=KEY            delete a particular key from the\n\
                                       key data base\n\
+  -T type, --output-type=type	      Select output type for quiet mode\n\
+                                      (text, html, tex or nsr)\n"
+#ifdef ENABLE_SAVE_TESTS				      
+"\
+  -s, --list-sessions		      List the sessions available on the remote\n\
+                                      host (must be used with -q)\n\
+  -R ID, --restore-session=ID	      Restore  the session <ID>\n"			      
+#endif  	      
+"\
   -q, --batch-mode                    "
 #ifdef USE_GTK
                                      "force an error if there is no\n\
@@ -359,8 +414,15 @@ printf ("\
                                      "ignored (compatibiltity mode, only)"
 #endif
 "\n\
-Argument list:\n\
-  SERVER PORT LOGIN TRG RESULT\n\
+Argument list:\n"
+#ifdef ENABLE_SAVE_TESTS
+"\
+  SERVER PORT LOGIN [TRG] [RESULT]\n"
+#else
+"\
+  SERVER PORT LOGIN TRG RESULT\n"
+#endif
+"\
     |     |    |     |   |\n\
     |     |    |     |   +-- file name where nessus stores the results, in\n\
     |     |    |     +------ file name containing the target host adresses\n\
@@ -392,17 +454,106 @@ Argument list:\n\
  printf("\t\tport     : nessusd host port\n");
 #endif
  printf("\t\tuser     : user name\n");
+ printf("\t\tpass      : password\n");
  printf("\t\ttargets  : file containing the list of targets\n");
  printf("\t\tresult   : name of the file where \n\t\t\t   nessus will store the results\n");
 #endif /* standard version */
 }
  
+/*
+ * version check (for libraries)
+ *
+ * Returns 0  if versions are equal
+ * Returns 1 if the fist version is newer than the second 
+ * Return -1 if the first version is older than the second
+ *
+ */
+static int 
+version_check(a,b)
+ char * a, *b;
+{
+ int major_a = 0, minor_a = 0, patch_a = 0;
+ int major_b = 0, minor_b = 0, patch_b = 0;
+ 
+ 
+ major_a = atoi(a);
+ a = strchr(a, '.');
+ if(a)
+ {
+  minor_a = atoi(a+sizeof(char));
+  a = strchr(a+sizeof(char), '.');
+  if(a)patch_a = atoi(a+sizeof(char));
+ }
+ 
+ major_b = atoi(b);
+ b = strchr(b, '.');
+ if(b)
+ {
+  minor_b = atoi(b+sizeof(char));
+  b = strchr(b+sizeof(char), '.');
+  if(b)patch_b = atoi(b+sizeof(char));
+ }
+ 
+ if(major_a < major_b)return -1;
+ if(major_a > major_b)return 1;
+ 
+ /* major are the same */
+ if(minor_a < minor_b)return -1;
+ if(minor_a > minor_b)return 1;
+ 
+ /* minor are the sames */
+ if(patch_a < patch_b)return -1;
+ if(patch_a > patch_b)return 1;
+ 
+ return 0;
+}
+
+	
+	
+
+
+
+
  
 int main(int argc, char * argv[])
 {
   int i, xac;
   char *myself, **xav;
   int gui = 1;
+  output_func_t output_func = NULL;
+#ifdef ENABLE_SAVE_TESTS  
+  int list_sessions = 0;
+  int restore_session = 0;
+  char * session_id = NULL;
+#endif  
+  /*
+   * Version check
+   */
+  if(version_check(NESSUS_VERSION, nessuslib_version())>0)
+  {
+   fprintf(stderr, 
+"Error : we are linked against nessus-libraries %s. \n\
+Install nessus-libraries %s or make sure that\n\
+you have deleted older versions nessus libraries from your system\n",
+        nessuslib_version(), NESSUS_VERSION);
+   exit (1);
+  }
+  
+#ifdef ENABLE_CRYPTO_LAYER  
+  /*
+   * We want peks 0.8.10 or newer
+   */
+  if(strncmp("peks/0.8", peks_version(), strlen("peks/0.8"))<0)
+  {
+   fprintf(stderr, 
+"Error : we are linked against libpeks %s. \n\
+Install libpeks %s or make sure that\n\
+you have deleted older versions of libpeks from your system\n",
+        peks_version(), "0.8");
+   exit(1);
+  }
+#endif
+  
   if ((myself = strrchr (*argv, '/')) == 0
 #ifdef _WIN32
       && (myself = strrchr (*argv, '\\')) == 0
@@ -411,10 +562,6 @@ int main(int argc, char * argv[])
   else
     myself ++ ;
 
-#ifndef DEBUG
-  signal(SIGSEGV , sighand_sigsegv);
-#endif
-
   PluginsNum = 0;
   ScannersNum = 0;
   Scanners = Plugins = MainDialog = NULL;
@@ -425,13 +572,17 @@ int main(int argc, char * argv[])
   F_show_pixmaps = 1;
 #endif
 
+#ifdef ENABLE_CRYPTO_LAYER
+  /* allow to ue another name for HOME */
+  peks_set_homevar ("NESSUSHOME");
+#endif
+
   /* provide a extra acrgc/argv vector for later use */
   xac = 1;
   xav = append_argv (0, myself);
   pty_logger ((void(*)(const char*, ...))printf);
 
   for (;;) {
-    int this_option_optind = optind ? optind : 1;
     int option_index = 0;
     static struct option long_options[] =
     {
@@ -447,17 +598,46 @@ int main(int argc, char * argv[])
 #endif
       {"batch-mode",           no_argument, 0, 'q'},
       {"config-file",		required_argument, 0, 'c'},
+      {"output-type",			required_argument, 0, 'T'},
+#ifdef ENABLE_SAVE_TESTS      
+      {"list-sessions",		no_argument, 0, 's'},
+      {"restore-session",       required_argument, 0, 'R'},
+#endif      
       {0, 0, 0, 0}
     };
 
     if ((i = getopt_long 
-	 (argc, argv, "c:vhqnk:?CLK:r:01", long_options, &option_index)) == EOF)
+	 (argc, argv, "c:T:vhqnk:?CLK:r:01sR:", long_options, &option_index)) == EOF)
       break;
      else
       
     switch(i) {
+     case 'T' :
+       if(!optarg)
+       {
+        display_help("nessus");
+	exit (1);
+       }
+        if(optarg[0]=='=')inc_optind(); /* no optind++ on Win32 -- jordan */
+	if(!strcmp(optarg, "text"))output_func = arglist_to_text;
+	else if(!strcmp(optarg, "html"))output_func = arglist_to_html;
+	else if(!strcmp(optarg, "tex"))output_func = arglist_to_latex;
+	else if(!strcmp(optarg, "latex"))output_func = arglist_to_latex;
+	else if(!strcmp(optarg, "nsr"))output_func = arglist_to_file;
+	else if(!strcmp(optarg, "xml"))output_func = arglist_to_xml;
+	else {
+		fprintf(stderr, "%s - invalid output type\n", optarg);
+		fprintf(stderr, "Valid types are : text, html, tex, xml and nsr\n");
+		exit (1);
+	     }
+	break;    
      case 'c' :
-       Alt_rcfile = estrdup(optarg);
+       if(!optarg)
+       {
+        display_help("nessus");
+	exit (1);
+       }
+       else Alt_rcfile = estrdup(optarg);
        break;
 #ifdef USE_GTK
     case 'n' : 
@@ -471,7 +651,7 @@ int main(int argc, char * argv[])
 #endif    	
 
     case 'v' :
-    	printf("nessus (%s) %s for %s\n\n(C) 1998-1999 Renaud Deraison <deraison@nessus.org>\n", 
+    	printf("nessus (%s) %s for %s\n\n(C) 1998, 1999, 2000 Renaud Deraison <deraison@nessus.org>\n", 
     			PROGNAME,NESSUS_VERSION, NESS_OS_NAME);
 #ifdef ENABLE_CRYPTO_LAYER
 	printf("\tsupports crypto layer version %s\n", peks_version ());
@@ -518,7 +698,19 @@ int main(int argc, char * argv[])
       gui = 0; 
       F_quiet_mode ++ ;
       break;
-
+#ifdef ENABLE_SAVE_TESTS
+    case 's' :
+      list_sessions ++;
+      break;
+    case 'R' :
+      restore_session ++;
+      if(optarg)session_id = strdup(optarg);
+      else {
+      	display_help(myself);
+	exit(1);
+	}
+       break;	
+#endif
     default:
       display_help (myself);
       exit (0);
@@ -526,35 +718,76 @@ int main(int argc, char * argv[])
   }
 
  if(!gui)F_quiet_mode = 1;
-#ifdef ENABLE_CRYPTO_LAYER
+ 
+#ifdef ENABLE_SAVE_TESTS
+ if(list_sessions && (argc<=optind) && !F_quiet_mode)
+ {
+  display_help(myself);
+  exit(1);
+ }
+ 
+ if(restore_session && (argc<=optind) && !F_quiet_mode)
+ {
+  display_help(myself);
+  exit(1);
+ }
+ 
+ if(restore_session && list_sessions)
+ {
+  fprintf(stderr, "--restore-session and --list-sessions are mutually exclusive\n");
+  exit(1);
+ }
+#endif
+
+
  if(argc>optind || F_quiet_mode)
      {
+      signal(SIGINT, sighand_exit);
+      signal(SIGQUIT, sighand_exit);
+      signal(SIGKILL, sighand_exit);
+      signal(SIGTERM, sighand_exit);
+      
+#ifdef ENABLE_CRYPTO_LAYER     
       auth_pwd = cmdline_pass;
+#endif      
       F_quiet_mode = 1;
      }
 #ifdef USE_GTK     
   else
+    {
       init_display (NULL, 0);
+#ifdef ENABLE_CRYPTO_LAYER      
+      if(!F_quiet_mode){
+	peks_set_keep_alive_notice(splash);
+	peks_get_pwd = splash_passphrase;
+	}
 #endif      
-      
+     }
+#endif
+  
+  /* system environment set up */
+  if (preferences_init(&Prefs))
+    exit (2);
+
+#ifdef ENABLE_CRYPTO_LAYER      
   key_initialization_stuff ();
 #endif
 
-  
-  /* system environment set up */
-  preferences_init(&Prefs);
-
   /* do we run in batchmode ? */
   if (argc > optind || F_quiet_mode) {
-    char *username, *target, *fname, *err, *cipher = "";
-#ifdef USE_AF_INET      
-    char *nessusd_host, *port;
-#endif
+    struct cli_args * cli;
+    struct arglist * hosts = emalloc(sizeof(struct arglist));
+ 
     F_quiet_mode = 1;
-
+    First_time = 1;
+    
+    cli = cli_args_new();
+    
 #ifdef ENABLE_CRYPTO_LAYER
-    cipher = "twofish/ripemd160:3";
+    cli_args_cipher(cli, DEFAULT_CIPHER);
     auth_pwd = cmdline_pass ; /* no gui, available */
+    cli_args_auth_pwd(cli, cmdline_pass);
+   
     /* implies ENABLE_CRYPTO_LAYER */
 #   define NUM_ARGS 5       
 #else
@@ -565,64 +798,95 @@ int main(int argc, char * argv[])
 #   undef  NUM_ARGS
 #   define NUM_ARGS 3
 #endif
+ 
 
+#ifndef ENABLE_SAVE_TESTS
     if (argc - optind != NUM_ARGS) {
       display_help(myself);
       exit(0);
     }
+#else
+ if(list_sessions)
+  {
+  if (argc - optind != NUM_ARGS - 2) {
+      display_help(myself);
+      exit(0);
+    }
+  }
+  else if(restore_session)
+  {
+    if (argc - optind != NUM_ARGS - 1) {
+      display_help(myself);
+      exit(0);
+    }
+  }
+  else
+  if (argc - optind != NUM_ARGS) {
+      display_help(myself);
+      exit(0);
+    }
+
+   
+#endif    
+    
 
     /* next arguments: SERVER PORT */
 #ifdef USE_AF_INET
-    nessusd_host = argv[inc_optind()];
-    port = argv[inc_optind()];
+    cli_args_server(cli, argv[inc_optind()]);
+    cli_args_port(cli, atoi(argv[inc_optind()]));
 #else
-    nessusd_host = "localhost";
-    port = 1;
+    cli_args_server(cli, "localhost");
+    cli_args_port(cli, 0);
 #endif
 
     /* next argument: LOGIN */
-    username = argv[inc_optind()];
-
+    cli_args_login(cli, argv[inc_optind()]);
+   
     /* next argument: PASSWORD */
 #ifndef ENABLE_CRYPTO_LAYER
-    cipher = argv[inc_optind()];
+    cli_args_password(cli, argv[inc_optind()]);
 #endif	
 
-    /* next argument: TARGET_FILE */
-    if ((target = target_file_to_list(argv[inc_optind()])) == 0)
-      exit(1);
-
-    /* next argument: RESULTS_FILE */
-    fname = argv[inc_optind()];
+#ifndef ENABLE_SAVE_TESTS
+    cli_args_target(cli, argv[inc_optind()]);
+    cli_args_results(cli,  argv[inc_optind()]);
+#else  
+   if(restore_session)
+   {
+     cli_args_results(cli,  argv[inc_optind()]);
+   }
+   else
+    if(!list_sessions)
+     {
+     cli_args_target(cli, argv[inc_optind()]);
+     cli_args_results(cli,  argv[inc_optind()]);
+     }
+   
+   
+#endif      
 
+    if(output_func)cli_args_output(cli, output_func);
+    else cli_args_best_output(cli);
+    
+    
     /* login now */
-    if((err = connect_to_nessusd
-	(nessusd_host, atoi(port), username, cipher))) {
-      fprintf(stderr, "%s : %s\n", myself, err);
-      exit(0);
-    }
-	
-    /* start attack */
-    {
-      int type, finished = 0;
-      char buf [4096], msg [4096];
-
-      attack_host(target, Prefs);
-      Hosts = emalloc(sizeof(struct arglist));   
-      
-      while(!finished){
-	network_gets(buf, 4095);
-	buf[strlen(buf)-1]=0;
-	if((type = parse_server_message(buf, Hosts, msg))==MSG_BYE)
-	  finished = 1;
-	bzero(msg, 4095);
-      }
-
-    }
-
-    /* store results */
-    arglist_to_file(Hosts, fname);
-
+    if((cli_connect_to_nessusd(cli))<0)
+	exit(1);
+#ifdef ENABLE_SAVE_TESTS
+    if(list_sessions){
+    	cli_list_sessions(cli);
+	shutdown(GlobalSocket,2);
+	close(GlobalSocket);
+	GlobalSocket = -1;
+	exit(0);
+	}
+    else if(restore_session) {
+    	cli_restore_session(cli, session_id);
+	}
+   else	
+#endif    	
+    cli_test_network(cli);
+    cli_report(cli);
     /* end, exit */
     exit(0);
   }
@@ -644,21 +908,20 @@ int main(int argc, char * argv[])
   
   for (i = 1; i < xac; i ++) {
     struct arglist * h = emalloc(sizeof(struct arglist));
-    file_to_arglist (h, xav [i]);
-    if(!Hosts) 
-      Hosts = h;
-    report_tests (h);
+    file_to_arglist (&h, xav [i]);
+    report_tests (h, 0);
    } 
 
   gtk_main();
   shutdown(GlobalSocket,2);
   close_display();
+  GlobalSocket = -1;
   return(0);
 
 #else
   printf("\nOoops ...\n\
-  This nessus version has no gui support.  You need to give nessus the
-  arguments SERVER PORT LOGIN TRG RESULT as explained in more detail
+  This nessus version has no gui support.  You need to give nessus the\n\
+  arguments SERVER PORT LOGIN TRG RESULT as explained in more detail\n\
   using the --help option.\n");
   exit (1);
 #endif
