@@ -23,13 +23,13 @@
 #include "preferences.h"
 #include "globals.h"
 #include "nessus.h"
-
+#include "error_dialog.h"
 static int preferences_new();
 static int prefs_buffer_parse(char *, struct arglist *);
 static int prefs_add_subcategory(struct arglist *, FILE *);
 static char * preferences_get_filename();
 static void new_pluginset(struct arglist *, struct arglist *);
-static int preferences_process(char *, struct arglist *);
+
 
 
 
@@ -44,6 +44,8 @@ int preferences_init(prefs)
 #ifndef ENABLE_CRYPTO_LAYER
   efree(&filename);
 #endif
+  if (result && getenv ("NESSUSHOME") == 0)
+    show_error_and_wait (CANNOT_SET_HOMEVAR);
   return(result);
 }
 
@@ -103,6 +105,56 @@ preferences_get_filename()
 #endif
  }
 }
+
+
+
+static char * 
+preferences_get_dirname()
+{
+#ifdef ENABLE_CRYPTO_LAYER
+  char *t, *s = peks_get_dirfile (NESSUS_RCFILE);
+  if (s == 0) 
+    return 0 ;
+  /* peks uses another memory management */
+  t = estrdup (s);
+  xfree (s);
+#ifdef _WIN32
+  /* on Windows we have no HOME varable set, by default */
+  if (getenv("HOME") == 0 && (s = peks_get_homedir (0)) != 0) {
+    char *e = strcat (strcpy (pmalloc (strlen (s) + 10), "HOME="), s);
+#   ifdef HAVE__PUTENV
+    _putenv (e);
+#   else
+    putenv (e);
+#   endif
+    xfree (e);
+  }
+#endif /* _WIN32 */
+  return t;
+#else
+  char* home;
+  char * ret;
+  struct passwd * pwd;
+  
+  home = getenv("HOME");
+  if(home)
+  {
+   ret = emalloc(strlen(home)+strlen("/.nessusrc")+1);
+   sprintf(ret, "%s/.nessusrc", home);
+   return(ret);
+  }
+  pwd = getpwuid(getuid());
+  if (pwd && pwd->pw_dir)
+  {
+   ret = emalloc(strlen(pwd->pw_dir)+strlen("/.nessusrc")+1);
+   sprintf(ret, "%s/.nessusrc", pwd->pw_dir);
+   return(ret);
+  } 
+  return(NULL);
+#endif
+}
+
+
  
 static int preferences_new()
 {
@@ -112,12 +164,13 @@ static int preferences_new()
  
   if(!(f = fopen(fn, "w")))
       {
-	perror("open ");
+        show_error_and_wait(strerror(errno));
 	return -1;
       }
   fprintf(f,"# Nessus Client Preferences File\n\n");
   fprintf(f, "begin(SCANNER_SET)\n");
   fprintf(f, "Ping the remote host = yes\n");
+  fprintf(f, "TCP Ping the remote host = no\n");
   fprintf(f, "TCP SYN scan = no\n");
   fprintf(f, "FTP bounce scan = no\n");
   fprintf(f, "Nmap tcp connect() scan = no\n");
@@ -125,11 +178,12 @@ static int preferences_new()
   fprintf(f, "end(SCANNER_SET)\n\n");
 
   fclose(f);
+  chmod(fn, 0600);
   efree(&fn);
   return(0);
 }
 
-static int preferences_process(filename,prefs)
+int preferences_process(filename,prefs)
      char * filename;
      struct arglist * prefs;
 {
@@ -137,12 +191,15 @@ static int preferences_process(filename,prefs)
   char * buffer;
     if(filename)
       {
+        chmod(filename, 0600);
 	if(!(fd = fopen(filename, "r"))) {
 #ifndef NESSUSNT
 	 if(errno == EACCES)
 	 {
-	  fprintf(stderr,
+	  char * buf = malloc(255 + strlen(filename));
+	  sprintf(buf,
 	  	"The Nessus client doesn't have the right to read %s\n", filename);
+	  show_error_and_wait(buf);
 	  exit(1);
 	 }
 #endif
@@ -150,17 +207,21 @@ static int preferences_process(filename,prefs)
 	  printf("Couldn't find any prefs file... Creating a new one...\n");
 #endif 
 	  if((preferences_new())<0){
-	    fprintf(stderr, "Error creating %s : ", preferences_get_filename());
-	    perror("open ");
+	    char * buf = malloc(255 + strlen(preferences_get_filename()) + strlen(strerror(errno)));
+	    sprintf(buf, "Error creating %s : %s", preferences_get_filename(), strerror(errno));
+	     show_error_and_wait(buf);
 	    return(1);
 	  }
 	  else
 	    if(!(fd = fopen(filename, "r")))
 	      {
-	        perror("open ");
-		printf("Could not open %s -- now quitting\n", filename);
-		exit(2);
-	      }
+	      char * buf = malloc(255 + strlen(preferences_get_filename()) + strlen(strerror(errno)));
+	      sprintf(buf, "Error creating %s : %s", preferences_get_filename(), strerror(errno));
+	      show_error_and_wait(buf);
+	      exit(2);
+	      }     
+
+		   
 	}
 	buffer = emalloc(255);
 	while(!feof(fd) && fgets(buffer, 254,fd))
@@ -254,12 +315,13 @@ prefs_buffer_parse(buffer, arglist)
  else return(1);
 }
 
+
 void
-preferences_save(plugins)
+preferences_save_fname(filename, plugins)
+ char * filename;
  struct arglist * plugins;
 {
  FILE * fd;
- char * filename = preferences_get_filename();
  struct arglist * t;
  fd = fopen(filename , "w");
  if(!fd)
@@ -268,6 +330,7 @@ preferences_save(plugins)
   efree(&filename);
   return;
   }
+ chmod(filename, 0600);
  fprintf(fd, "# This file was automagically created by nessus\n");
  t = Prefs;
  while(t && t->next)
@@ -294,9 +357,9 @@ preferences_save(plugins)
       v = v->next;
       continue;
       }
-    if((int)v->type==ARG_INT)
+    if(v->type==ARG_INT)
      fprintf(fd, " %s = %s\n", v->name, v->value?"yes":"no");
-    else if((v->type == ARG_STRING)&&(strlen(v->value)))
+    else if((v->type == ARG_STRING)&&v->value && strlen(v->value))
     	fprintf(fd, " %s = %s\n", v->name, (char *)v->value);
      v = v->next;
    }
@@ -306,18 +369,28 @@ preferences_save(plugins)
  }
   fclose(fd);
 }
-
-
 void
+preferences_save(plugins)
+ struct arglist * plugins;
+{
+ preferences_save_fname(preferences_get_filename(), plugins);
+}
+
+
+int
 pluginset_apply(plugins, name)
  struct arglist * plugins;
  char * name;
 {
+ int ret = 0;
  while(plugins && plugins->next)
  {
   int flag = 0;
   struct arglist * t = arg_get_value(Prefs, name);
-  if(!t)return;
+  
+  
+  if(!t)return 1;
+  
   while(!flag && t && t->next)
   {
    if(!strcmp(plugins->name, t->name))
@@ -327,8 +400,17 @@ pluginset_apply(plugins, name)
     }
     t = t->next;
    }
+   /* we do not know anything about this current plugin */
+   if(!flag)
+   {
+    char * c = arg_get_value(plugins->value, "CATEGORY");
+    if(c && !strcmp(c, "denial"))plug_set_launch(plugins->value, 0);
+    else plug_set_launch(plugins->value, 1);
+    ret++;
+   }
    plugins = plugins->next;
   }
+  return ret;
  }
  
 void
@@ -403,18 +485,5 @@ static void new_pluginset(pluginset, plugins)
 void prefs_check_defaults(prefs)
  struct arglist * prefs;
 {
- if(!arg_get_value(prefs, "port_range"))
- {
-  char * range = emalloc(12);
-  sprintf(range, "1-8000");
-  arg_add_value(prefs, "port_range", ARG_STRING, strlen(range), range);
- }
- 
- if(!arg_get_value(prefs, "max_threads"))
- {
-  char * max = emalloc(3);
-  sprintf(max, "15");
-  arg_add_value(prefs, "max_threads", ARG_STRING, strlen(max), max);
- }
 }
  
