@@ -33,9 +33,15 @@
 #include "prefs_dialog.h"
 #include "prefs_about.h"
 
+#ifdef ENABLE_SAVE_KB
+#include "prefs_kb.h"
+#endif
+
+
 #include "../error_dialog.h"
 #include "../monitor_dialog.h"
 #include "../report.h"
+#include "../read_target_file.h"
 #include "globals.h"
 
 static void prefs_dialog_set_tooltips(struct arglist *);
@@ -64,6 +70,9 @@ prefs_dialog_setup(widget, preferences)
  struct arglist * prefs_auth;
  struct arglist * prefs_plugins;
  struct arglist * prefs_plugins_prefs;
+#ifdef ENABLE_SAVE_KB 
+ struct arglist * prefs_kb;
+#endif 
  struct arglist * prefs_about;
  struct arglist * ctrls = emalloc(sizeof(struct arglist));
 
@@ -97,6 +106,7 @@ prefs_dialog_setup(widget, preferences)
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
  gtk_box_pack_start(GTK_BOX(box), notebook, TRUE, TRUE, 0);
  gtk_widget_show(notebook);
+ arg_add_value(ctrls, "NOTEBOOK", ARG_PTR, -1, notebook);
 
 /*
  * We set up the pages of our notebook
@@ -107,6 +117,9 @@ prefs_dialog_setup(widget, preferences)
  prefs_auth = prefs_dialog_auth(window);
  prefs_plugins = prefs_dialog_plugins();
  prefs_plugins_prefs = prefs_dialog_plugins_prefs();
+#ifdef ENABLE_SAVE_KB
+ prefs_kb = prefs_dialog_kb(); 
+#endif 
  prefs_about  = prefs_dialog_about(window);
  
  arg_add_value(ctrls, "SCAN_OPTIONS", ARG_ARGLIST, -1, prefs_scan);
@@ -116,6 +129,9 @@ prefs_dialog_setup(widget, preferences)
  arg_add_value(ctrls, "AUTH", ARG_ARGLIST, -1, prefs_auth);
  arg_add_value(ctrls, "PLUGINS", ARG_ARGLIST, -1, prefs_plugins);
  arg_add_value(ctrls, "PLUGINS_PREFS", ARG_ARGLIST, -1, prefs_plugins_prefs);
+#ifdef ENABLE_SAVE_KB
+ arg_add_value(ctrls, "SAVE_KB", ARG_ARGLIST, -1, prefs_kb);
+#endif 
  arg_add_value(ctrls, "ABOUT", ARG_ARGLIST, -1, prefs_about);
 /*
  * and we append them to it
@@ -126,10 +142,6 @@ prefs_dialog_setup(widget, preferences)
  gtk_widget_show(frame);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
  
- label = gtk_label_new("Target selection");
- frame = arg_get_value(prefs_target, "FRAME");
- gtk_widget_show(frame);
- gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
 
  label = gtk_label_new("Plugins");
  frame = arg_get_value(prefs_plugins, "FRAME");
@@ -157,11 +169,25 @@ prefs_dialog_setup(widget, preferences)
 		     prefs_scan);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
  
+ 
+
+ label = gtk_label_new("Target selection");
+ frame = arg_get_value(prefs_target, "FRAME");
+ gtk_widget_show(frame);
+ gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
+
  label = gtk_label_new("User");
  frame = arg_get_value(prefs_user, "FRAME");
  gtk_widget_show(frame);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
 
+#ifdef ENABLE_SAVE_KB
+ label = gtk_label_new("KB");
+ frame = arg_get_value(prefs_kb, "FRAME");
+ gtk_widget_show(frame);
+ gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
+#endif
+
  label = gtk_label_new("Credits");
  frame = arg_get_value(prefs_about, "FRAME");
  gtk_widget_show(frame);
@@ -235,11 +261,19 @@ void prefs_dialog_set_defaults(ctrls, preferences)
   w = arg_get_value(t, "DNS_EXPAND");
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(w), (flag & EX_DNS));
  }
- gtkw = arg_get_value(t, "MAX_HOSTS");
- v = arg_get_value(serv, "max_hosts");
- 
- if(v && !strcmp(v, "-1"))gtk_entry_set_text(GTK_ENTRY(gtkw), "infinity");
- else gtk_entry_set_text(GTK_ENTRY(gtkw), v);
+
+#ifdef ENABLE_SAVE_TESTS
+  gtkw = arg_get_value(t, "SAVE_THIS_SESSION");
+  v = arg_get_value(serv, "save_session");
+  if(arg_get_type(serv, "save_session") == ARG_INT)
+  {
+      char * s = strdup(v ? "yes":"no");
+      arg_set_type(serv, "save_session", ARG_STRING);
+      arg_set_value(serv, "save_session", strlen(s), s);
+      v = s;
+  } 
+  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw),(v && !strcmp(v, "yes")));
+#endif 
  
  /*
   * Scan options 
@@ -289,32 +323,73 @@ void prefs_dialog_set_defaults(ctrls, preferences)
   v = arg_get_value(serv, "optimize_test");
   if(v)gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw), !strcmp(v, "yes"));
   else gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw), 1);
+ 
+#ifdef ENABLE_SAVE_KB
+
+  /*
+   * Detached scan and continuous scan are ALWAYS
+   * disabled
+   */
+  gtkw = arg_get_value(t, "DETACHED_SCAN");
+  v = arg_get_value(serv, "detached_scan");
+  if(arg_get_type(serv, "detached_scan")==ARG_INT)
+  {
+   char * s = strdup("no");
+   
+   arg_set_value(serv, "detached_scan", strlen(s), s);
+   arg_set_type(serv, "detached_scan", ARG_STRING);
+  }
+  
+  v = arg_get_value(serv, "detached_scan");
+  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw), 0);
+  
+  gtkw = arg_get_value(t, "CONTINUOUS_SCAN");
+  v = arg_get_value(serv, "continuous_scan");
+  if(arg_get_type(serv, "continuous_scan")==ARG_INT)
+  {
+   char * s = strdup("no");
+   
+   arg_set_value(serv, "continuous_scan", strlen(s), s);
+   arg_set_type(serv, "continuous_scan", ARG_STRING);
+  }
+  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw), 0);
+  
+
+  v = arg_get_value(serv, "delay_between_scan_loops");
+  if(v)
+  {
+   gtkw = arg_get_value(t, "DELAY");
+   gtk_entry_set_text(GTK_ENTRY(gtkw), v);
+  }
+#endif  
+ 
+ 
   v = arg_get_value(serv, "port_range");
   if(v)
   {
    gtkw = arg_get_value(t, "PORT_RANGE");
-   gtk_entry_set_text(GTK_ENTRY(gtkw), v);
+   if(gtkw)gtk_entry_set_text(GTK_ENTRY(gtkw), v);
   }
  
   gtkw = arg_get_value(t, "MAX_THREADS");
-   
-  v = arg_get_value(serv, "max_threads");
-  if(v)gtk_entry_set_text(GTK_ENTRY(gtkw), v);
-   
+  if(gtkw)
+  {
+   v = arg_get_value(serv, "max_threads");
+   if(v)gtk_entry_set_text(GTK_ENTRY(gtkw), v);
+  }
    
    gtkw = arg_get_value(t, "CGI_PATH");
-   
-   v = arg_get_value(serv, "cgi_path");
-   if(v)gtk_entry_set_text(GTK_ENTRY(gtkw), v);
-   
+   if(gtkw)
+   {
+    v = arg_get_value(serv, "cgi_path");
+    if(v)gtk_entry_set_text(GTK_ENTRY(gtkw), v);
+   }
    /*
     * User
     */
     
     t = arg_get_value(ctrls, "USER");
-    gtkw = arg_get_value(t, "EMAIL");
-    v = arg_get_value(serv, "email");
-    if(v)gtk_entry_set_text(GTK_ENTRY(gtkw), v);
+ 
     gtkw = arg_get_value(t, "RULES");
     if(gtkw)
     {
@@ -351,6 +426,11 @@ void prefs_dialog_set_defaults(ctrls, preferences)
       arg_add_value(t, "RULES_DLIST", ARG_PTR, -1, dlist);
      gtk_list_append_items(GTK_LIST(gtkw), dlist);
     }
+    
+#ifdef ENABLE_SAVE_KB
+   prefs_dialog_kb_set_prefs(arg_get_value(ctrls, "SAVE_KB"),
+   			     preferences);
+#endif    
 }
   
   
@@ -371,7 +451,7 @@ static void prefs_dialog_set_tooltips(ctrls)
 #endif
 #ifdef ENABLE_CRYPTO_LAYER
  w = arg_get_value(t, "CIPHER");
- gtk_tooltips_set_tip(tooltips, w, HLP_CIPHER_CHOICE,"");
+ if(w)gtk_tooltips_set_tip(tooltips, w, HLP_CIPHER_CHOICE,"");
 #endif
  w = arg_get_value(t, "USERNAME");
  gtk_tooltips_set_tip(tooltips, w, HLP_LOGIN_USER,"");
@@ -379,8 +459,7 @@ static void prefs_dialog_set_tooltips(ctrls)
  t = arg_get_value(ctrls, "TARGET");
  w = arg_get_value(t, "DNS_EXPAND");
  gtk_tooltips_set_tip(tooltips, w, HLP_HOST_EXPANSION_DNS,"");
- w = arg_get_value(t, "MAX_HOSTS");
- gtk_tooltips_set_tip(tooltips, w, HLP_HOST_EXPANSION_MAX_HOSTS, "");
+ 
  w = arg_get_value(t, "TARGET");
  gtk_tooltips_set_tip(tooltips, w, HLP_TARGET_PRIMARY_TARGET, "");
  
@@ -390,6 +469,9 @@ static void prefs_dialog_set_tooltips(ctrls)
  gtk_tooltips_set_tip(tooltips, w, HLP_SCAN_OPT_PING,"");
  w = arg_get_value(t, "OPTIMIZE_TEST");
  gtk_tooltips_set_tip(tooltips, w, HLP_SCAN_OPT_OPTIMIZE, "");
+#ifdef ENABLE_KB
+#warn "No tooltips for detached scan!"
+#endif
  w = arg_get_value(t, "REVERSE_LOOKUP");
  gtk_tooltips_set_tip(tooltips, w, HLP_SCAN_OPT_REVERSE_LOOKUP, "");
  w = arg_get_value(t, "PORT_RANGE");
@@ -399,8 +481,6 @@ static void prefs_dialog_set_tooltips(ctrls)
  w = arg_get_value(t, "CGI_PATH");
  gtk_tooltips_set_tip(tooltips, w, HLP_CGI_PATH, "");
  t = arg_get_value(ctrls, "USER");
- w = arg_get_value(t, "EMAIL");
- gtk_tooltips_set_tip(tooltips, w, HLP_USER_EMAIL, "");
  gtk_tooltips_enable(tooltips);
 }
 
@@ -440,9 +520,22 @@ int prefs_dialog_ok(w, ctrls)
    if(!strlen(expansion))strncpy(expansion, "none;", 5);
    expansion[strlen(expansion)-1]=0;
    
+#ifdef ENABLE_SAVE_TESTS   
+  gtkw = arg_get_value(t, "SAVE_THIS_SESSION");
+  if(GTK_TOGGLE_BUTTON(gtkw) -> active)
+  	s = strdup("yes");
+  else
+  	s = strdup("no");
+
+  if(arg_get_value(serv, "save_session"))
+   arg_set_value(serv, "save_session", strlen(s), s);
+  else
+   arg_add_value(serv, "save_session", ARG_STRING, strlen(s), s);
+#endif 
+   
    s = gtk_entry_get_text(GTK_ENTRY(arg_get_value(t, "TARGET")));
-   hostname = emalloc(strlen(s)+1);
-   strncpy(hostname, s, strlen(s));
+   hostname = target_translate(s); /* if the target is a file, then
+   				      translate it */
    
    s = emalloc(strlen(expansion)+1);
    strncpy(s, expansion, strlen(expansion));
@@ -453,26 +546,7 @@ int prefs_dialog_ok(w, ctrls)
     arg_add_value(serv, "host_expansion", ARG_STRING, strlen(s), s);
     
 
-  gtkw = arg_get_value(t, "MAX_HOSTS");
-  s = gtk_entry_get_text(GTK_ENTRY(gtkw));
-  if(!strcmp(s, "infinity"))
-  {
-   s = emalloc(3);
-   strncpy(s, "-1", 2);
-  }
-  else
-  {
-   s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(gtkw)))+1);
-   strncpy(s, gtk_entry_get_text(GTK_ENTRY(gtkw)), 
- 	strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))));
-  }
-  
-  
-  if(arg_get_value(serv, "max_hosts"))
-   arg_set_value(serv, "max_hosts", strlen(s), s);
-  else
-   arg_add_value(serv, "max_hosts", ARG_STRING, strlen(s), s);
-  
+ 
   /*
    * Plugins preferences
    */
@@ -492,7 +566,7 @@ int prefs_dialog_ok(w, ctrls)
      if((pref = arg_get_value(plugins->value, "plugin_prefs")))
       while(pref && pref->next)
       {
-       if((type = arg_get_value(pref->value, "type")))
+       if((type = (char*)arg_get_value(pref->value, "type")))
        {
         if(!strcmp(type, PREF_ENTRY))
          {
@@ -524,7 +598,7 @@ int prefs_dialog_ok(w, ctrls)
         {
           GtkWidget * button = list->data;
 	  if(GTK_TOGGLE_BUTTON(button)->active)
-	   value = gtk_object_get_data(GTK_OBJECT(button), "name");
+	   value = (char*)gtk_object_get_data(GTK_OBJECT(button), "name");
 	  list = list->next;
         }
         else {
@@ -583,7 +657,40 @@ int prefs_dialog_ok(w, ctrls)
    else
     arg_add_value(serv, "optimize_test", ARG_STRING, strlen(s), s);
   
+#ifdef ENABLE_SAVE_KB
+   gtkw = arg_get_value(t, "DETACHED_SCAN");
+   if(GTK_TOGGLE_BUTTON(gtkw)->active){
+   	 s = strdup("yes");
+	 DetachedMode = 1;
+	}
+   else {
+   	 s = strdup("no");
+    	 DetachedMode = 0;
+        }
+   if(arg_get_value(serv, "detached_scan"))
+    arg_set_value(serv, "detached_scan", strlen(s), s);
+   else
+    arg_add_value(serv, "detached_scan", ARG_STRING, strlen(s), s);  
   
+   gtkw = arg_get_value(t, "CONTINUOUS_SCAN");
+  
+   
+   if(GTK_TOGGLE_BUTTON(gtkw)->active)s = strdup("yes");
+   else s = strdup("no");
+   
+   if(arg_get_value(serv, "continuous_scan"))
+    arg_set_value(serv, "continuous_scan", strlen(s), s);
+   else
+    arg_add_value(serv, "continuous_scan", ARG_STRING, strlen(s), s);  
+   
+   gtkw = arg_get_value(t, "DELAY");
+   s = strdup(gtk_entry_get_text(GTK_ENTRY(gtkw)));
+   if(arg_get_value(serv, "delay_between_scan_loops"))
+    arg_set_value(serv, "delay_between_scan_loops", strlen(s), s);
+   else
+    arg_add_value(serv, "delay_between_scan_loops", ARG_STRING, strlen(s), s); 
+    
+#endif     
  gtkw = arg_get_value(t, "PORT_RANGE");
  s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(gtkw)))+1);
  strncpy(s, gtk_entry_get_text(GTK_ENTRY(gtkw)), 
@@ -617,6 +724,12 @@ int prefs_dialog_ok(w, ctrls)
     arg_set_value(serv, "cgi_path", strlen(s), s);
   else
     arg_add_value(serv, "cgi_path", ARG_STRING, strlen(s), s);   
+
+#ifdef ENABLE_SAVE_KB
+  t = arg_get_value(ctrls, "SAVE_KB");
+  prefs_dialog_kb_get_prefs(t);
+#endif
+
   
  /* 
   * User
@@ -626,18 +739,7 @@ int prefs_dialog_ok(w, ctrls)
   
   
  t = arg_get_value(ctrls, "USER");
- gtkw = arg_get_value(t, "EMAIL");
- s = gtk_entry_get_text(GTK_ENTRY(gtkw));
- if(s)
- {
-  e = emalloc(strlen(s)+1);
-  strncpy(e, s, strlen(s));
-  if(arg_get_value(serv, "email"))
-   arg_set_value(serv, "email", strlen(e), e);
-  else
-   arg_add_value(serv, "email",ARG_STRING, strlen(e), e);
- }
- else e = NULL;
+ e = NULL;
  
   /*
    * Check for the errors
@@ -657,25 +759,14 @@ a scan in the 'nessusd' section");
 to attack in the 'target' section");
    return(0);
   }
-  
-  if(!e && !arg_get_value(serv, "email"))
-  {
-   show_error("You must enter your email address in \n\
-the 'user' section");
-   return(0);
-  }
-  
-  
-  /*
-   * Save the prefs to file
-   */
-  preferences_save(Plugins);
+ 
+
   
   /*
    * Set up the GUI for the attack, and start it !
    */
   close_window(w, arg_get_value(ctrls, "WINDOW"));
-  monitor_dialog_setup(hostname);
+  monitor_dialog_setup(hostname, 0);
   
   return(0);
 }
