@@ -37,7 +37,8 @@
 extern char * stored_pwd;
 
 static char * get_username();
-static int prefs_dialog_auth_callback(GtkWidget* , struct arglist *);
+static int prefs_dialog_login_callback(GtkWidget* , struct arglist *);
+static int prefs_dialog_logout_callback(GtkWidget* , struct arglist *);
 struct arglist * prefs_dialog_auth();
 
 #ifndef USE_AF_INET      
@@ -91,7 +92,7 @@ struct arglist * prefs_dialog_auth(window)
  GtkWidget * separator;
  GtkWidget * entry;
  char * default_server = arg_get_value(Prefs, "nessusd_host");
- 
+ char * default_user = arg_get_value(Prefs, "nessusd_user");
     
   /*
    * Set up the main frame
@@ -170,7 +171,7 @@ struct arglist * prefs_dialog_auth(window)
   gtk_widget_show(separator);
   
   entry = gtk_entry_new();
-  gtk_entry_set_text (GTK_ENTRY(entry), get_username ());
+  gtk_entry_set_text (GTK_ENTRY(entry), default_user?default_user:get_username ());
   
 #ifdef ENABLE_CRYPTO_LAYER
   gtk_table_attach_defaults(GTK_TABLE(table), entry, 1,2,4,5);
@@ -201,21 +202,35 @@ struct arglist * prefs_dialog_auth(window)
 #endif /* ENABLE_CRYPTO_LAYER */
 
 
-  button = gtk_button_new_with_label(" Log in now ! ");
+  button = gtk_button_new_with_label("Log in");
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
 #ifdef ENABLE_CRYPTO_LAYER
   gtk_table_attach(GTK_TABLE(table), button, 1,2,7,8,GTK_FILL | GTK_EXPAND,0,10,10);
 #else
   gtk_table_attach(GTK_TABLE(table), button, 1,2,6,7,GTK_FILL | GTK_EXPAND,0,10,10);
 #endif /* ENABLE_CRYPTO_LAYER */
-  arg_add_value(ctrls, "BUTTON", ARG_PTR, -1, button);
+  arg_add_value(ctrls, "BUTTON_LOG_IN", ARG_PTR, -1, button);
   gtk_signal_connect /*_object*/(GTK_OBJECT (button), "clicked",
- 			   (GtkSignalFunc)prefs_dialog_auth_callback,
+ 			   (GtkSignalFunc)prefs_dialog_login_callback,
  			   (void *)ctrls);
                            
   gtk_widget_grab_default (button);
   gtk_widget_show(button);
   
+  button = gtk_button_new_with_label(" Log out");
+  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+#ifdef ENABLE_CRYPTO_LAYER
+  gtk_table_attach(GTK_TABLE(table), button, 1,2,7,8,GTK_FILL | GTK_EXPAND,0,10,10);
+#else
+  gtk_table_attach(GTK_TABLE(table), button, 1,2,6,7,GTK_FILL | GTK_EXPAND,0,10,10);
+#endif /* ENABLE_CRYPTO_LAYER */
+  arg_add_value(ctrls, "BUTTON_LOG_OUT", ARG_PTR, -1, button);
+  gtk_signal_connect /*_object*/(GTK_OBJECT (button), "clicked",
+ 			   (GtkSignalFunc)prefs_dialog_logout_callback,
+ 			   (void *)ctrls);
+                           
+
+  
   label = gtk_label_new(" Connected");
 #ifdef ENABLE_CRYPTO_LAYER
   gtk_table_attach(GTK_TABLE(table), label, 0,1,6,7, GTK_FILL | GTK_EXPAND, 0,10,10);
@@ -267,7 +282,20 @@ struct arglist * prefs_dialog_auth(window)
   return(ctrls);
 }
 
-static int prefs_dialog_auth_callback(w, ctrls)
+static int prefs_dialog_logout_callback(w, ctrls)
+ GtkWidget * w;
+ struct arglist * ctrls;
+{
+ shutdown(GlobalSocket, 2);
+ closesocket(GlobalSocket);
+ GlobalSocket = -1;
+ 
+ gtk_widget_hide(arg_get_value(ctrls, "BUTTON_LOG_OUT"));
+ gtk_widget_show(arg_get_value(ctrls, "BUTTON_LOG_IN"));
+ gtk_widget_hide(arg_get_value(ctrls, "CONNECTED"));
+ return 0;
+}
+static int prefs_dialog_login_callback(w, ctrls)
  GtkWidget * w;
  struct arglist * ctrls;
 {
@@ -279,12 +307,14 @@ static int prefs_dialog_auth_callback(w, ctrls)
 #endif
  char * t;
  char * err;
-  
+#ifdef ENABLE_CRYPTO_LAYER 
+ int hostkey = 0;
+#endif 
 #ifdef USE_AF_INET
  t = gtk_entry_get_text(GTK_ENTRY(arg_get_value(ctrls, "HOSTNAME")));
  if((!t) ||(!strlen(t)))
  {
-  show_error("You must enter an hostname");
+  show_warning("You must enter an hostname");
   return(1);
  }
  hostname = emalloc(strlen(t)+1);
@@ -292,20 +322,20 @@ static int prefs_dialog_auth_callback(w, ctrls)
  t = gtk_entry_get_text(GTK_ENTRY(arg_get_value(ctrls, "PORT")));
  if((!t) ||(!strlen(t)))
    {
-     show_error("You must enter a valid port number !");
+     show_warning("You must enter a valid port number !");
      return(1);
    }
  port = atoi(t);
  if((port < 0) || (port > 65536))
    {
-     show_error("Your port specification is illegal");
+     show_warning("Your port specification is illegal");
      return(1);
    }
 #endif
   t = gtk_entry_get_text(GTK_ENTRY(arg_get_value(ctrls, "USERNAME")));
   if((!t) ||(!strlen(t)))
     {
-      show_error("You must enter a valid username");
+      show_warning("You must enter a valid username");
       return(1);
     }
   username = emalloc(strlen(t)+1);
@@ -314,7 +344,7 @@ static int prefs_dialog_auth_callback(w, ctrls)
   t = gtk_entry_get_text(GTK_ENTRY(arg_get_value(ctrls, "PASSWORD")));
   if((!t) ||(!strlen(t)))
     {
-      show_error("You must enter a valid password");
+      show_warning("You must enter a valid password");
       return(1);
     }
   password = emalloc(strlen(t)+1);
@@ -331,6 +361,8 @@ static int prefs_dialog_auth_callback(w, ctrls)
     return (1) ;
   }
   stored_pwd = password;
+  
+tryagain: 
   err = connect_to_nessusd(hostname, port, username, t);
 #else
   err = connect_to_nessusd(hostname, port, username, password);
@@ -338,12 +370,32 @@ static int prefs_dialog_auth_callback(w, ctrls)
 #else
   err = connect_to_nessusd("localhost", -1, username, password);
 #endif
-  if(err)show_error(err);
+  if(err){
+  	/* XXX fixme because I'm ugly ! */
+        if(!strcmp(err, "Host key for that server has been saved"))
+	{
+#ifdef ENABLE_CRYPTO_LAYER
+         show_info(err);
+  	 if(!hostkey)
+	 {
+	  hostkey++;
+	  goto tryagain;
+	 }
+#endif
+	}
+	else
+	  show_error(err);
+	}
  else
  {
- gtk_widget_hide(arg_get_value(ctrls, "BUTTON"));
+ gtk_widget_hide(arg_get_value(ctrls, "BUTTON_LOG_IN"));
+ gtk_widget_show(arg_get_value(ctrls, "BUTTON_LOG_OUT"));
  gtk_widget_show(arg_get_value(ctrls, "CONNECTED"));
-
+ 
+ /*
+  * Go to the plugins page
+  */
+ gtk_notebook_set_page(GTK_NOTEBOOK(arg_get_value(MainDialog, "NOTEBOOK")), 1);
  if(First_time==0)
  {
   prefs_plugins_redraw(NULL,NULL,arg_get_value(MainDialog, "PLUGINS"));
