@@ -1,5 +1,5 @@
 /* Nessus
- * Copyright (C) 1999 Renaud Deraison
+ * Copyright (C) 1999, 2000 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -114,7 +114,7 @@ prefs_plugins_reset(ctrls, plugins, scanners)
  struct arglist * scanners;
 {
  struct arglist * prefs;
- struct arglist * s[2] = {plugins, scanners};
+ struct arglist * s[2] = {scanners, plugins};
  GtkWidget * frame;
  GtkWidget * s_window;
  GtkWidget * vbox;
@@ -161,7 +161,7 @@ prefs_plugins_reset(ctrls, plugins, scanners)
     */
    v = prefs->value;
    v = v->next;
-   arg_free(v->next);
+  /* arg_free(v->next); */
    v->next = emalloc(sizeof(struct arglist));
    prefs = prefs->next;
    }
@@ -185,8 +185,8 @@ prefs_plugins_prefs_redraw(bidon1, bidon2, ctrls)
  if((num != PluginsNum)||(num2 != ScannersNum))
  {
  prefs_plugins_reset(ctrls, Plugins, Scanners);
- prefs_dialog_plugins_prefs_fill(ctrls, Plugins);
  prefs_dialog_plugins_prefs_fill(ctrls, Scanners);
+ prefs_dialog_plugins_prefs_fill(ctrls, Plugins);
  arg_set_value(ctrls, "PLUGINS_NUM", sizeof(int), (void *)PluginsNum);
  arg_set_value(ctrls, "SCANNERS_NUM", sizeof(int), (void*)ScannersNum);
  }
@@ -229,7 +229,22 @@ static void
  GtkWidget * vbox = arg_get_value(ctrls, "VBOX");
  GtkWidget * entry, * text, * box;
  char * name = pref->name;
+ char * fullname = arg_get_value(pref->value, "fullname");
+ struct arglist * pprefs = arg_get_value(Prefs, "PLUGINS_PREFS");
  
+ if(pprefs)
+ {
+  int type;
+  if((type = arg_get_type(pprefs, fullname))>=0)
+  {
+   value = arg_get_value(pprefs, fullname);
+   if(type==ARG_INT)
+    {
+    if(value)value=strdup("yes");
+    else value = strdup("no");
+    }
+  }
+ }
  box = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, FALSE, 5);
  gtk_widget_show(box);
@@ -253,10 +268,27 @@ static void
 {
  GtkWidget * vbox = arg_get_value(ctrls, "VBOX");
  GtkWidget * orig;
- GtkWidget * button;
+ GtkWidget * button, * first_button;
  GtkWidget * label;
  char * t;
  GSList * list = NULL;
+ char * fullname = arg_get_value(pref->value, "fullname");
+ struct arglist * pprefs = arg_get_value(Prefs, "PLUGINS_PREFS");
+ char * def = NULL;
+ if(pprefs)
+ {
+  int type;
+  if((type = arg_get_type(pprefs, fullname))>=0)
+  {
+   def = arg_get_value(pprefs, fullname);
+   if(type==ARG_INT)
+    {
+    if(def)def=strdup("yes");
+    else def = strdup("no");
+    }
+  }
+ }
+ 
  
  label = gtk_label_new(estrdup(pref->name));
  gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, FALSE, 5);
@@ -264,12 +296,15 @@ static void
  
  t = strchr(value, ';');
  if(t)t[0] = '\0';
- orig = gtk_radio_button_new_with_label(NULL, value);
+ first_button = orig = gtk_radio_button_new_with_label(NULL, value);
  gtk_box_pack_start(GTK_BOX(vbox), orig, TRUE, FALSE, 5);
  gtk_object_set_data(GTK_OBJECT(orig), "name", value);
  gtk_widget_show(orig);
- gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(orig), TRUE);
-
+ if(def)
+ {
+ gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(orig),TRUE);
+ }
+ else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(orig),TRUE);
  value = t+sizeof(char);
  if(t)
     while(value)
@@ -279,13 +314,17 @@ static void
            gtk_radio_button_group(GTK_RADIO_BUTTON(orig)), 
 	   value);
       gtk_object_set_data(GTK_OBJECT(button), "name", value);
-      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), FALSE);
+      if(def && !strcmp(def, value))
+       {
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),TRUE);
+	}
       gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, FALSE, 5);
       gtk_widget_show(button);
       if(t)value = t+sizeof(char);
       else value = NULL;
      }
  list = gtk_radio_button_group(GTK_RADIO_BUTTON(orig));
+
  arg_add_value(pref->value, "RADIOBUTTONS", ARG_PTR, -1, list);
 }
 
@@ -299,7 +338,22 @@ static void
  GtkWidget * box;
  GtkWidget * button;
  char * name = pref->name;
- 
+ struct arglist * pprefs = arg_get_value(Prefs, "PLUGINS_PREFS");
+ char * def = NULL;
+ char * fullname = arg_get_value(pref->value, "fullname");
+ if(pprefs)
+ {
+  int type;
+  if((type = arg_get_type(pprefs, fullname))>=0)
+  {
+   def = arg_get_value(pprefs, fullname);
+   if(type==ARG_INT)
+    {
+    if(def)def=strdup("yes");
+    else def = strdup("no");
+    }
+  }
+ }
  box = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, FALSE, 5);
  gtk_widget_show(box);
@@ -307,6 +361,14 @@ static void
  button = gtk_check_button_new_with_label(estrdup(name));
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 5);
  gtk_widget_show(button);
+ if(def)
+ {
+  if(!strcmp(def, "yes"))
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);
+  else
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), FALSE);
+  }
+ else
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), !strcmp(value, "yes"));
  arg_add_value(pref->value, "CHECKBOX", ARG_PTR, -1, button);
 }
