@@ -96,7 +96,8 @@ auth_check_user(globals, from)
     if (l  &&  buf_password[l-1] == '\r')buf_password[--l] = '\0';
   }
 
-  if((permissions = check_user(usersdb, buf_user, buf_password)))
+  if((permissions = check_user(usersdb, buf_user, buf_password))
+     && (permissions != BAD_LOGIN_ATTEMPT))
   {
 	char* user = emalloc(strlen(buf_user)+1);
 	strncpy(user, buf_user, strlen(buf_user));
