@@ -121,6 +121,15 @@ ntp_caps* comm_init(soc)
       caps->pubkey_auth = TRUE;
       send(soc, "< NSP/0.2 >\n", 12, 0);
    }
+  else if(!strncmp(buf, "< NSP/0.3 >", 11))
+  {
+      caps->ntp_version = NSP_01;
+      caps->ciphered = TRUE;
+      caps->ntp_11 = TRUE;
+      caps->scan_ids = TRUE;
+      caps->pubkey_auth = TRUE;
+      send(soc, "< NSP/0.3 >\n", 12, 0);
+  }
 # endif
   else
     {
@@ -159,11 +168,11 @@ void
 comm_send_pluginlist(globals)
  struct arglist * globals;
 {
-  int i=1,j, k;
+  int i=1,j;
   const char * categories[] =
   	{"scanner", "infos", "attack", "denial", "sniffer", "unknown" };
   struct arglist * plugins = arg_get_value(globals, "plugins");
-  struct arglist * w = NULL;
+ 
   
   auth_printf(globals, "SERVER <|> PLUGIN_LIST <|>\n");
   while(plugins && plugins->next)
@@ -174,7 +183,8 @@ comm_send_pluginlist(globals)
       
 	  
       args = arg_get_value(plugins->value, "plugin_args");
-      arg_add_value(args, "ID", ARG_INT, sizeof(int), (void *)i);
+      if(!arg_get_value(args, "ID"))
+        arg_add_value(args, "ID", ARG_INT, sizeof(int), (void *)i);
       t = arg_get_value(args, "DESCRIPTION");
       if(t)while((t=strchr(t,'\n')))t[0]=';';
       j = (int)arg_get_value(args, "CATEGORY");
@@ -189,12 +199,30 @@ comm_send_pluginlist(globals)
 	  a = "unknown NAME" ;
 	log_write ("Inconsistent data: %s - not applying this plugin\n", a);
       } else
+      {
+       if(strchr(a, '\n')){
+       	fprintf(stderr, "ERROR - %d %s\n", (int)arg_get_value(args, "ID"), a);
+	}
 	
+	if(strchr(b, '\n')){
+       	fprintf(stderr, "ERROR - %d %s\n", (int)arg_get_value(args, "ID"), b);
+	
+	}
+	
+	if(strchr(c, '\n')){
+       	fprintf(stderr, "ERROR - %d %s\n", (int)arg_get_value(args, "ID"), c);
+	
+	}
+	
+	if(strchr(d, '\n')){
+       	fprintf(stderr, "ERROR - %d %s\n", (int)arg_get_value(args, "ID"), d);
+	}
       auth_printf(globals, "%d <|> %s <|> %s <|> %s <|> %s <|> %s <|> %s\n",
-      		  i, a,
+      		  (int)arg_get_value(args, "ID"), a,
 		  categories[j-1],
 		  b, c, d,
-		  plug_get_family(args));
+		  (char*)plug_get_family(args));
+      }
       i++;
       plugins = plugins->next;
     }
@@ -208,9 +236,10 @@ void
 comm_send_rules(globals)
  struct arglist * globals;
 {
- struct nessus_rules * rules = arg_get_value(globals, "rules");
+ 
  auth_printf(globals, "SERVER <|> RULES <|>\n");
 #ifdef USELESS_AS_OF_NOW
+ struct nessus_rules * rules = arg_get_value(globals, "rules");
  while(rules && rules->next)
  {
   char c = 0;
@@ -245,6 +274,9 @@ comm_send_preferences(globals)
   auth_printf(globals, "%s <|> %s\n", prefs->name, (const char *) prefs->value);
   prefs = prefs->next;
  }
+#ifdef ENABLE_SAVE_TESTS
+ auth_printf(globals, "ntp_save_sessions <|> yes\n");
+#endif
  auth_printf(globals, "<|> SERVER\n");
 }
 
@@ -270,7 +302,7 @@ comm_wait_order(globals)
 
   for (;;) {
     char str [2048] ;
-    /* bzero(str, sizeof(str)); */
+   
     recv_line (soc, str, sizeof (str)-1);
     if (str [0] == '\0') {
 #     ifdef ENABLE_CRYPTO_LAYER
@@ -289,16 +321,13 @@ comm_wait_order(globals)
       EXIT(0);
 #     endif /* ENABLE_CRYPTO_LAYER */
     }
-    /* else */
-    /* { */
-    /* loop condition: end or continue */
+   
     if(caps->ntp_11) {
       if (ntp_11_parse_input (globals, str) == 0) break ;
       continue ;
     }
-    /* else */ 
     if (ntp_10_parse_input (globals, str) == 0) break ;
-    /* } */
+    
   }
 
 # ifdef ENABLE_CRYPTO_LAYER
@@ -321,7 +350,6 @@ comm_setup_plugins(globals, id_list)
   char * list = id_list;
   char * buf;
   int id;
-  int i = 0;
   int num_plugins=0;
   struct arglist * plugins = arg_get_value(globals, "plugins");
   struct arglist * p = plugins;
@@ -347,7 +375,6 @@ comm_setup_plugins(globals, id_list)
       if(id<0)
  	{
 	  struct arglist * lp = NULL;
-          int i;
           
           lp=plugins;
 	  while(lp && lp->next)
@@ -357,14 +384,16 @@ comm_setup_plugins(globals, id_list)
 	    }
 	  break;
  	}
-      if(id>num_plugins)
-	{
-	  auth_printf(globals, 
-          	"SERVER <|> ERROR <|> There is no plugin of id %d <|> SERVER\n", id);
-	  p = NULL;
-	}
-      if(p)for(i=1;i!=id;i++)p = p->next;
-      if(p)plug_set_launch(p->value, 1);
+       while(p && p->next)
+	 {
+	  struct arglist * pa = arg_get_value(p->value, "plugin_args");
+	  int pid = (int)arg_get_value(pa, "ID");
+	  if(id == pid){
+	  	plug_set_launch(p->value, 1);
+		break;
+		}
+	  p = p->next;
+	 }
       sprintf(list, "%s", buf);
       bzero(buf, strlen(id_list)+1);
     }
