@@ -30,6 +30,7 @@
 #include "crypto-layer.h"
 #include "log.h"
 #include <sys/types.h>
+#include "sighand.h"
 
 #ifdef HAVE_SELECT
  /* stdout, implements sleep () using select () */
@@ -135,7 +136,12 @@ verify_root_access
 
   /* we check the real id despite the fact that nessusd can run suid */
   print_error("Only root may access the user data base - aborting.\n");
+
+# ifdef DEBUG
+  fprintf(stderr, "   (Note: Debug mode does not exit, here)\n");
+# else
   DO_EXIT (1);
+# endif
 }
 
 
@@ -251,13 +257,13 @@ cipher_stuff_init
    struct arglist *prefs)
 {
   peks_key *key;
-  const char *user, *rpcu, *file ;
+  const char *user, *rpcu, *file, *devrnd ;
   int klen ;
  
   /* overwrite defaults by config */
          arg_reorg_s2i  (prefs, "peks_pwdfail",  NESSUSD_MAXPWDFAIL) ;
   klen = arg_reorg_s2i  (prefs, "peks_keylen",   NESSUSD_KEYLENGTH) ;
-  
+   
   /* publish client negotiation timeout */
          arg_reorg_s2i  (prefs, "negot_timeout", NESSUSD_NEGOT_TIMEOUT) ;
  
@@ -270,6 +276,11 @@ cipher_stuff_init
   user = arg_reorg_s2ptr (prefs, "peks_username", NESSUSD_USERNAME) ;
   rpcu = arg_reorg_s2ptr (prefs, "peks_rpcuser",  user);
 
+  if ((devrnd = (const char*)arg_get_value (prefs, "random_device")) != 0) {
+    if (strcasecmp (devrnd, "none") == 0) devrnd = 0 ;
+    use_random_device (devrnd) ;
+  }
+
   if ((key = peks_private_key (user, file, get_pwd, klen)) == 0) {
     print_error("%s - aborting.\n", peks_strerr (errno));
     DO_EXIT (1);
@@ -311,20 +322,31 @@ do_local_cipher_commands (opts, prefs)
      struct arglist * prefs;
 {
   char *pwd_user =     arg_get_value (opts,  "pwd_user");
+  char *list_upwd =    arg_get_value (opts,  "list_user_pwd");
   char *pwd_passwd   = arg_get_value (opts,  "pwd_passwd");
   char *kill_userkey = arg_get_value (opts,  "kill_userkey");
   int chng_pph =  (int)arg_get_value (opts,  "chng_pph");
   int list_udb =  (int)arg_get_value (opts,  "list_udb");
   peks_key *key      = arg_get_value (prefs, "peks_key");
+  char * s;
 
   /* no connection, but user passwd table etc. request */
   int did_something = 0 ;
   
+  /* password table entry list request */
+  if (list_upwd != 0) {
+    verify_root_access ();
+    if ((s = peks_edit_passwd_file
+	 (key, list_upwd, -1, 0, arg_get_value (prefs, "peks_usrkeys"))) == 0) {
+      print_error("ERROR: %s.\n", peks_strerr (errno));
+      DO_EXIT (1);
+    }
+    printf ("passwd specs for %s: %s\n", list_upwd, s);
+    did_something ++ ;
+  }
+
   /* password table edit request */
   if (pwd_user != 0) {
-    peks_key *key = arg_get_value (prefs, "peks_key");
-    char * s;
-
     verify_root_access ();
   
     /* delete entry ? */
@@ -332,7 +354,6 @@ do_local_cipher_commands (opts, prefs)
       pwd_passwd = 0; 
       key = 0;
     }
-
     if ((s = peks_edit_passwd_file
 	 (key, pwd_user, 0, pwd_passwd,
 	  arg_get_value (prefs, "peks_usrkeys"))) == 0) {
@@ -342,7 +363,6 @@ do_local_cipher_commands (opts, prefs)
 
     if (s [0])
       printf ("%s\n", s) ;
-
     did_something ++ ;
   }
  
@@ -394,7 +414,7 @@ verify_io_process (unused, pid)
     return 1;
 
   /* let that process be killed */
-  log_write ("Process %u has died without closing IO threads.\n", pid);
+  log_write ("Process %lu has died without closing IO threads.\n", pid);
   return 0 ;
 }
 
