@@ -23,6 +23,8 @@
 #include <includes.h>
 #include <nessus/nasl.h>
 #include "pluginload.h"
+#include "preferences.h"
+#include "threads.h"
 /*
  *  Initialize the nasl system
  */
@@ -31,8 +33,22 @@ static pl_class_t* nasl_plugin_init(struct arglist* prefs,
     return &nasl_plugin_class;
 }
 
+
+static int current_nasl_plugin = -1;
+
 static void nasl_thread(struct arglist *);
 
+static 
+void nasl_plugin_sigterm()
+{
+ if(current_nasl_plugin >= 0)
+ {
+  kill(current_nasl_plugin, SIGKILL);
+  current_nasl_plugin = -1;
+ }
+ EXIT(0);
+}
+
 /*
  *  Add *one* .nasl plugin to the plugin list
  */
@@ -43,12 +59,11 @@ nasl_plugin_add(folder, name, plugins, preferences)
      struct arglist * plugins;
      struct arglist * preferences;
 {
- ext_library_t ptr = NULL; 
  char * fullname = NULL;
  harglst * nasl = init_nasl(preferences_get_checks_read_timeout(preferences));
  harglst *  nasl_globals;
  harglst *  nasl_types;
- struct arglist * args, *plugin_args;
+ struct arglist *plugin_args;
  struct arglist * plugin;
  char * lang = "english";
  
@@ -104,8 +119,8 @@ nasl_plugin_launch(globals, plugin, hostinfos, preferences, kb, name)
  struct arglist * args;
  
  int timeout;
- int category;
  int pip[2];
+ int category = 0;
  nthread_t module;
  
  harglst * nasl = init_nasl(preferences_get_checks_read_timeout(preferences));
@@ -114,13 +129,16 @@ nasl_plugin_launch(globals, plugin, hostinfos, preferences, kb, name)
  struct arglist * d = emalloc(sizeof(struct arglist));
  short* ports = arg_get_value(globals, "ports");
  int ports_num = (int)arg_get_value(globals, "ports_num");
- fd_set rd;
- struct timeval tv;
  char * buf = emalloc(512);
  socketpair(AF_UNIX, SOCK_STREAM, 0, pip);
  args = arg_get_value(plugin, "plugin_args");
  arg_add_value(args, "HOSTNAME", ARG_ARGLIST, -1, hostinfos);
- arg_add_value(args, "globals", ARG_ARGLIST, -1, globals);
+ if(arg_get_value(args, "globals"))
+   arg_set_value(args, "globals", -1, globals);
+ else    
+   arg_add_value(args, "globals", ARG_ARGLIST, -1, globals);
+ 
+ 
  arg_add_value(args, "ports", ARG_PTR, -1, ports);
  arg_add_value(args, "ports_num", ARG_INT, sizeof(int), (void*)ports_num);
  arg_set_value(args, "preferences", -1, preferences);
@@ -137,9 +155,12 @@ nasl_plugin_launch(globals, plugin, hostinfos, preferences, kb, name)
  arg_add_value(d, "name", ARG_STRING, -1, name);
  
  if(category == ACT_SCANNER)timeout = -1;
- else if(!timeout)timeout = PLUGIN_TIMEOUT;  
- module = create_thread(nasl_thread, d, globals); 
- thread_timeout(module, timeout, pip[0], kb);
+ else if(!timeout)timeout = preferences_plugins_timeout(preferences);  
+ 
+ current_nasl_plugin = module = create_thread(nasl_thread, d, (struct arglist*)-1); 
+ signal(SIGTERM, nasl_plugin_sigterm);
+ thread_timeout(globals, module, timeout, pip[0], kb);
+ current_nasl_plugin = -1;
  free(buf);
  close(pip[0]);
  close(pip[1]);
