@@ -21,6 +21,64 @@
 
 #include <includes.h>
 #include "pluginload.h"
+#include "threads.h"
+#include "log.h"
+#include "preferences.h"
+/*
+ * Kill the subthreads
+ */
+
+static int current_nes_plugin = -1;
+
+static
+void nes_plugin_sigterm()
+{
+ if(current_nes_plugin >= 0)
+ {
+ kill(current_nes_plugin, SIGTERM);
+ current_nes_plugin = -1;
+ }
+ EXIT(0);
+}
+
+#ifdef HAVE_SHL_LOAD	/* this is HP/UX */
+ext_library_ptr dlopen(name)
+ char* name;
+{
+ return (ext_library_ptr)shl_load(name, BIND_IMMEDIATE|BIND_VERBOSE|BIND_NOSTART, 0L);
+}
+
+void * 
+dlsym(p, name)
+ shl_t lib;
+ char * name;
+{
+ void* ret;
+ int status;
+ status = shl_findsym((shl_t *)&lib, name, TYPE_PROCEDURE, &ret);
+ if((status == -1) && (errno == 0))
+ {
+  status = shl_findsym((shl_t *)&lib, name, TYPE_DATA, &ret);
+ }
+ 
+ return (status == -1 ) ? NULL : ret;
+}
+
+void
+dlclose(x)
+ shl_t * x;
+{
+ shl_unload(x);
+}
+
+char*
+dlerror()
+{
+ return strerror(errno);
+}
+
+#endif /* HAVE_SHL_LOAD */
+
 
 /*
  *  Initialize this class
@@ -29,7 +87,6 @@ pl_class_t* nes_plugin_init(struct arglist* prefs, struct arglist* args) {
     return &nes_plugin_class;
 }
 
-
 /*
  * add *one* .nes (shared lib) plugin to the server list
  */
@@ -42,7 +99,7 @@ nes_plugin_add(folder, name, plugins, preferences)
 {
  ext_library_t ptr = NULL; 
  char * fullname = NULL;
- struct arglist * plugin;
+ struct arglist * plugin = NULL;
  
 #ifndef NESSUSNT
   fullname = emalloc(strlen(folder)+strlen(name)+2);
@@ -62,18 +119,14 @@ nes_plugin_add(folder, name, plugins, preferences)
   }
   else {
     plugin_init_t  f_init;
-    struct arglist * p_args;
     struct arglist * args;
     plugin = emalloc(sizeof(struct arglist));   
     if((f_init = (plugin_init_t)LOAD_FUNCTION(ptr, "plugin_init")) ||
     	(f_init = (plugin_init_t)LOAD_FUNCTION(ptr, "_plugin_init")))
       {
-      	p_args = emalloc(sizeof(struct arglist));
-      	arg_add_value(p_args, "preferences", ARG_ARGLIST, -1, (void*)preferences);
-      	(*f_init)(p_args);
-        args = emalloc(sizeof(struct arglist));
-        arg_dup(args, p_args);
-        arg_free(p_args); 
+      	args = emalloc(sizeof(struct arglist));
+      	arg_add_value(args, "preferences", ARG_ARGLIST, -1, (void*)preferences);
+      	(*f_init)(args);
 	if(arg_get_value(args, "NAME"))
 	{
  	 arg_add_value(plugin, "plugin_args", ARG_ARGLIST, -1, (void *)args);
@@ -106,10 +159,8 @@ nes_plugin_launch(globals, plugin, hostinfos, preferences, kb, name)
 {
  struct arglist * args;
  int timeout;
- int category;
+ int category = 0;
  int pip[2];
- fd_set rd;
- struct timeval tv;
  char * buf = emalloc(512);
  short* ports = arg_get_value(globals, "ports");
  int ports_num =(int)arg_get_value(globals, "ports_num");
@@ -122,7 +173,7 @@ nes_plugin_launch(globals, plugin, hostinfos, preferences, kb, name)
  func = (plugin_run_t)LOAD_FUNCTION(ptr, "plugin_run");
  if(!func)func = (plugin_run_t)LOAD_FUNCTION(ptr, "_plugin_run");
  if(!func){
- 	log_write(globals, "no 'plugin_run()' function in %s\n", name);
+ 	log_write("no 'plugin_run()' function in %s\n", name);
 	return;
 	}
  socketpair(AF_UNIX, SOCK_STREAM, 0, pip);
@@ -137,9 +188,13 @@ nes_plugin_launch(globals, plugin, hostinfos, preferences, kb, name)
  timeout = (int)arg_get_value(args, "TIMEOUT");
  
  if(category == ACT_SCANNER)timeout = -1;
- else if(!timeout)timeout = PLUGIN_TIMEOUT;
- module = create_thread(func, args, globals);
- thread_timeout(module, timeout, pip[0], kb);
+ else if(!timeout)timeout = preferences_plugins_timeout(preferences);
+ 
+ 
+ current_nes_plugin = module = create_thread(func, args, (struct arglist*)-1);
+ signal(SIGTERM, nes_plugin_sigterm);
+ thread_timeout(globals, module, timeout, pip[0], kb);
+ current_nes_plugin = -1;
  free(buf);
  close(pip[0]);
  close(pip[1]);
