@@ -1,5 +1,5 @@
 /* Nessus
- * Copyright (C) 1998 Renaud Deraison
+ * Copyright (C) 1998, 1999, 2000 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,23 +17,29 @@
  */
  
 #include <includes.h>
-#ifndef NESSUSNT
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#else  
-extern int optind;
-extern char * optarg;
-#endif /* not HAVE_GETOPT_H */   
 
-#else/* not def NESSUSNT */
+
+#ifdef USE_LIBWRAP
+#include <tcpd.h>
+#include <syslog.h>
+
+int deny_severity = LOG_WARNING;
+int allow_severity = LOG_NOTICE;
+
+#endif
+
+
+#ifdef NESSUSNT
 #include "wstuff.h"
 #endif
 
+
 #ifdef ENABLE_CRYPTO_LAYER
 #include "crypto-layer.h"
 #endif
 
 
+#include "nessus/getopt.h"
 #include "pluginload.h"
 #include "preferences.h"
 #include "auth.h"
@@ -56,6 +62,8 @@ int single_shot = 0 ;
 #define setsid() setpgrp()
 #endif
 
+extern char * nasl_version();
+extern char * nessuslib_version();
 /*
  * Globals that should not be touched
  */
@@ -66,7 +74,7 @@ struct arglist * g_plugins;
 struct arglist * g_preferences;
 struct nessus_rules * g_rules;
 struct users * g_users;
-
+int Delay_between_tests;
 /*
  * Functions prototypes
  */
@@ -76,12 +84,29 @@ static int init_network(int, int *, struct in_addr);
 static void server_thread(struct arglist *);
 
 
+
+
+static void
+arg_replace_value(arglist, name, type, length, value)
+ struct arglist * arglist;
+ char * name;
+ int type;
+ int length;
+ void * value;
+{
+ if(arg_get_type(arglist, name)<0)
+  arg_add_value(arglist, name, type, length, value);
+ else  
+  arg_set_value(arglist, name, length, value);
+}
+
+
 static void
 start_daemon_mode
   (void)
 {
   char *s;
-  int fd, n;
+  int fd;
 
 # ifdef DEBUG
   if (single_shot)
@@ -106,7 +131,7 @@ start_daemon_mode
   
   /* no input, anypmore: provide an empty-file substitute */
   if ((fd = open ("/dev/null", O_RDONLY)) < 0) {
-    log_write ("Cannot open /dev/null -- aborting",strerror (errno));
+    log_write ("Cannot open /dev/null (%s) -- aborting",strerror (errno));
     exit (0);
   }
   dup2 (fd, 0);
@@ -123,6 +148,9 @@ start_daemon_mode
 	("Cannot open dumpfile %s (%s)-- aborting", s, strerror (errno));
       exit (2);
     }
+    chmod(s, 0600);
+    chown(s, getuid (), getgid ());
+	
     if (peks_private_access (s, 2)) {
       log_write ("Access rights problem with %s (%s)-- aborting", 
 		 s, peks_strerr (errno));
@@ -136,10 +164,11 @@ start_daemon_mode
       exit (2);
     }
 #   endif /* ENABLE_CRYPTO_LAYER */
+
     dup2 (fd, 1);
     dup2 (fd, 2);
     close (fd);
-    log_write ("Redirecting debugging output to %s", s);
+    log_write ("Redirecting debugging output to %s", s);  
   }
 }
 
@@ -164,11 +193,15 @@ sighup(i)
 {
   struct arglist * options = g_options;
   log_write("Caught HUP signal - reconfiguring nessusd\n");
+  plugins_free(g_plugins);
+  log_close();
   init_nessusd(options,0);
   g_plugins = arg_get_value(options, "plugins");
   g_preferences = arg_get_value(options, "preferences");
   g_rules = arg_get_value(options, "rules");
   g_users = arg_get_value(options, "users");
+  log_close();
+  log_init(arg_get_value(g_preferences, "logfile"));
 }
 
 
@@ -179,17 +212,18 @@ server_thread(globals)
 {
  struct sockaddr_in * address = arg_get_value(globals, "client_address");
  struct arglist * plugins = arg_get_value(globals, "plugins");
-#ifdef ENABLE_CRYPTO_LAYER
  struct arglist * prefs = arg_get_value (globals, "preferences") ;
+#ifdef ENABLE_CRYPTO_LAYER
  char * opt = NULL;
- int fd, n ;
+ int n ;
 #endif
  int soc = (int)arg_get_value(globals, "global_socket");
  struct nessus_rules* perms;
  char * asciiaddr;
  struct nessus_rules * rules = arg_get_value(globals, "rules");
  ntp_caps* caps;
- int sz = 4096;
+ 
+ 
 #ifdef USE_PTHREADS
  int off = 0;
  ioctl(soc, FIONBIO, &off);
@@ -198,6 +232,11 @@ server_thread(globals)
 #endif
 #endif
 	
+ /*
+  * Close the server thread - it is useless for us now
+  */
+ close(g_serv_socket);
+ 
 #ifdef HAVE_ADDR2ASCII
  asciiaddr = emalloc(20);
  addr2ascii(AF_INET, &address->sin_addr, sizeof(struct in_addr), asciiaddr);
@@ -244,8 +283,8 @@ server_thread(globals)
 #endif /* ENABLE_CRYPTO_LAYER */
 
  arg_add_value(globals, "ntp_caps", ARG_STRUCT, sizeof(*caps), caps);
- plugins_set_ntp_caps(plugins, caps);
- plugins_set_socket(plugins, soc);
+
+ 
  if(((perms = auth_check_user(globals, asciiaddr))==BAD_LOGIN_ATTEMPT)||
    !perms)
  {
@@ -273,7 +312,8 @@ server_thread(globals)
 #endif /* ENABLE_CRYPTO_LAYER */
    
    if(perms){
-     	rules_add(&rules, &perms);
+     	rules_add(&rules, &perms, NULL);
+	rules_set_client_ip(rules, address->sin_addr);
 #ifdef DEBUG_RULES
 	printf("Rules have been added : \n");
 	rules_dump(rules);
@@ -290,18 +330,35 @@ server_thread(globals)
    log_user_cookie (globals, "login", 0, 0) ;
 #endif /* ENABLE_CRYPTO_LAYER */
 
+  /* become process group leader and the like ... */
+
+
+  start_daemon_mode ();
+   
+   plugins_set_ntp_caps(plugins, caps);
+wait :   
+#ifdef ENABLE_SAVE_TESTS
+   if(arg_get_value(globals, "RESTORE-SESSION"))
+     arg_set_value(globals, "RESTORE-SESSION", sizeof(int),(void*)2);
+   else
+     arg_add_value(globals, "RESTORE-SESSION", ARG_INT, sizeof(int),(void*)2); 
+#endif       
    comm_wait_order(globals);
    rules = arg_get_value(globals, "rules");
-
-   /* become process group leader and the like ... */
-   start_daemon_mode ();
+   Delay_between_tests = preferences_get_delay_between_tests(prefs);
 
 #ifdef ENABLE_CRYPTO_LAYER
-   io_ctrl (soc, IO_RESIZE_BUF, (n=1024, &n), 0) ; /* limit read buffer */
+   io_ctrl (soc, IO_RESIZE_BUF, (n=4096, &n), 0) ; /* limit read buffer */
 #endif
-   attack_host(globals);
+
+   attack_network(globals);
    comm_terminate(globals);
-  
+   if(arg_get_value(prefs, "ntp_keep_communication_alive"))
+    {
+   	log_write("user %s : Kept alive connection",
+			(char*)arg_get_value(globals, "user"));
+   	goto wait;
+    }
 #ifdef ENABLE_CRYPTO_LAYER_currently_disabled
    log_user_cookie (globals, "terminated", 0, 0) ;
 #endif /* ENABLE_CRYPTO_LAYER */
@@ -323,11 +380,12 @@ main_loop()
 {
   char *old_addr = 0, *asciiaddr = 0;
   time_t last = 0;
-  int count ;
-
+  int count = 0;
 #ifdef ENABLE_CRYPTO_LAYER
+#ifdef ENABLE_CRYPTO_LAYER_currently_disabled  
   int lid = 0 ;
   char * logp = arg_get_value (g_preferences,"cookie_logpipe");
+#endif /* disabled */
 #ifdef USE_PTHREADS
   /* pre-initialize iostream data */
   io_ptinit (0);
@@ -348,6 +406,8 @@ main_loop()
       struct nessus_rules * my_rules;
       struct sockaddr_in * p_addr;
      
+     
+      
       /* prevent from an io table overflow attack against nessus */
       if (asciiaddr != 0) {
 	time_t now = time (0);
@@ -381,9 +441,18 @@ main_loop()
       old_addr = asciiaddr ;
       asciiaddr = 0 ;
       
+       
       soc = accept(g_serv_socket, (struct sockaddr *)(&address), &lg_address);
       if(soc == -1)continue;
       asciiaddr = estrdup(inet_ntoa(address.sin_addr));
+#ifdef USE_LIBWRAP      
+      if(!(hosts_ctl("nessusd", STRING_UNKNOWN, asciiaddr, STRING_UNKNOWN)))
+      {
+       shutdown(soc, 2);
+       close(soc);
+       continue;
+      }
+#endif      
       log_write("connection from %s\n", (char *)asciiaddr);
       /* efree(&asciiaddr); */
       /* 
@@ -435,7 +504,9 @@ main_loop()
       arg_add_value(globals, "rules", ARG_PTR, -1, my_rules);
 
       /* we do not want to create an io thread, yet so the last argument is -1 */
+     
       create_thread(server_thread, globals, (struct arglist *)(-1));
+      close(soc);
 #ifdef USE_FORK_THREADS
       arg_free(globals);
 #endif    
@@ -536,10 +607,11 @@ init_nessusd(options, first_pass)
   char * config_file = arg_get_value(options, "config_file");
   struct in_addr * addr = arg_get_value(options, "addr");
   preferences_init(config_file, &preferences);
-  /*
   if(arg_get_value(preferences, "max_threads"))
-   g_max_threads = atoi(arg_get_value(preferences, "max_threads"));
- */
+   {
+    g_max_threads = atoi(arg_get_value(preferences, "max_threads"));
+    if(!g_max_threads)g_max_threads = 15;
+   } 
 #ifdef ENABLE_CRYPTO_LAYER
   cipher_stuff_init        (options, preferences) ;
   do_local_cipher_commands (options, preferences) ;
@@ -578,50 +650,65 @@ init_nessusd(options, first_pass)
    nessus_signal(SIGPIPE, SIG_IGN);
   }
 #endif  
-   arg_add_value(options, "sock", ARG_INT, sizeof(int),(void *)sock);
-   arg_add_value(options, "plugins", ARG_ARGLIST, -1, plugins);
-   arg_add_value(options, "rules", ARG_PTR, -1, rules);
-   arg_add_value(options, "users", ARG_PTR, -1, users);
-   arg_add_value(options, "preferences", ARG_ARGLIST, -1, preferences);
+   arg_replace_value(options, "sock", ARG_INT, sizeof(int),(void *)sock);
+   arg_replace_value(options, "plugins", ARG_ARGLIST, -1, plugins);
+   arg_replace_value(options, "rules", ARG_PTR, -1, rules);
+   arg_replace_value(options, "users", ARG_PTR, -1, users);
+   arg_replace_value(options, "preferences", ARG_ARGLIST, -1, preferences);
    return(0);
 }
 
 
 static void 
-display_help()
+display_help 
+  (char *pname)
 {
-  printf("nessusd, version %s\n", NESSUS_VERSION);
+#ifdef ENABLE_CRYPTO_LAYER /* => USE_AF_INET */
+  printf("\nUsage: %s [Options]\n", pname); printf ("\n\
+Options:\n\
+  -D, --background                  runs in daemon mode\n\
+  -c FILE, --config-file=FILE       use another configuration file instead\n\
+                                    of " NESSUSD_CONF "\n\
+  -a BIND, --listen=BIND            listen on TCP address BIND rather than\n\
+                                    any address, possible\n\
+  -p PORT, --port=PORT              use port number PORT instead of %u\n\
+", DEFAULT_PORT); printf ("\n\
+  -v, --version                     display version number and exit\n\
+  -h, --help                        show this help text\n\
+  -d, --dump-cfg                    dump the compilation options\n\
+"); printf ("\n\
+  -C, --change-pass-phrase          interactively change the pass phrase\n\
+                                    for the private key\n\
+  -L, --list-keys                   list all keys in the data base\n\
+  -K KEY, --delete-key=KEY          delete a particular key from the\n\
+                                    key data base\n\
+  -U USER, --list-user-pwd=USER     list temporary user passwd and specs\n\
+  -P USER,PWD, --make-user=USER,PWD create or replace temp user and passwd\n\
+  -P USER,, --make-user=USER,       delete temporary user and passwd\n\
+  -P USER, --make-user=USER         list temporary user passwd\n\
+");
+#else /* ENABLE_CRYPTO_LAYER */
+
 #ifdef USE_AF_INET
-#ifdef ENABLE_CRYPTO_LAYER
-  printf("\nusage : nessusd [options]\n\noptions:\n");
-#else
+  printf("nessusd, version %s\n", NESSUS_VERSION);
   printf("\nusage : nessusd [-vcphdDLC] [-a address]\n\n");
-#endif /* ENABLE_CRYPTO_LAYER */
 #else
   printf("\nusage : nessusd [-vchdD]\n\n");
 #endif /*def USE_AF_INET */
-
   printf("\ta <address>    : listen on <address>\n");
   printf("\tv              : shows version number\n");
   printf("\th              : shows this help\n");
-
 #ifdef USE_AF_INET
   printf("\tp <number>     : use port number <number>\n");
 #endif /* USE_AF_INET */
-#ifdef ENABLE_CRYPTO_LAYER
-  printf("\tC              : change passphrase for private key\n");
-  printf("\tP <user>       : list temporary user passwd\n");
-  printf("\tP <user>,<pwd> : replace temporary user passwd\n");
-  printf("\tP <user>,      : delete temporary user passwd\n");
-  printf("\tK <user>       : kill registered user auth key\n");
-  printf("\tL              : list user auth key entries\n");
- #endif /* ENABLE_CRYPTO_LAYER */
   printf("\tc <filename>   : alternate configuration file to use\n");
   printf("\t\t\t (default : %s)\n", NESSUSD_CONF);
   printf("\tD              : runs in daemon mode\n");
   printf("\td              : dumps the nessusd compilation options\n");
+#endif /*  ENABLE_CRYPTO_LAYER */
 }
 
+
 #ifdef NESSUSNT
 int WINAPI WinMain(HINSTANCE hThisInst, HINSTANCE hPrevInst,
                                  LPSTR lpszArgs, int nWinMode)
@@ -632,6 +719,7 @@ main(int argc, char * argv[])
 {
   int serv_port = -1;
   char * config_file = NULL;
+  char * myself;
   int do_fork = 0;
   struct in_addr addr; 
   struct arglist * options = emalloc(sizeof(struct arglist));
@@ -642,11 +730,65 @@ main(int argc, char * argv[])
   struct rlimit rl = {0,0};
   setrlimit(RLIMIT_CORE, &rl);
 #endif
+
+  if ((myself = strrchr (*argv, '/')) == 0
+#ifdef NESSUSNT
+      && (myself = strrchr (*argv, '\\')) == 0
+#endif
+      ) myself = *argv ;
+  else
+    myself ++ ;
+
   if(getuid())
   {
-    printf("Only root should start nessusd.\n");
+     fprintf(stderr, "Only root should start nessusd.\n");
+#    ifdef DEBUG
+     fprintf(stderr, "   (Note: Debug mode does not exit, here)\n");
+#    else
      exit(1);
+#    endif
   }
+  
+  /*
+   * Version check
+   */
+
+  if(version_check(NESSUS_VERSION, nessuslib_version())>0)
+  {
+   fprintf(stderr, 
+"Error : we are linked against nessus-libraries %s. \n\
+Install nessus-libraries %s or make sure that\n\
+you have deleted older versions nessus libraries from your system\n",
+	nessuslib_version(), NESSUS_VERSION);
+   exit (1);
+  }
+
+
+  if(version_check(NESSUS_VERSION, nasl_version())>0)
+  {
+   fprintf(stderr, 
+"Error : we are linked against libnasl %s. \n\
+Install libnasl %s or make sure that\n\
+you have deleted older versions of libnasl from your system\n",
+	nasl_version(), NESSUS_VERSION);
+   exit (1);
+  }
+  
+  
+#ifdef ENABLE_CRYPTO_LAYER  
+  /*
+   * We want peks 0.8.10 or newer
+   */
+  if(strncmp("peks/0.8", peks_version(), strlen("peks/0.8"))<0)
+  {
+   fprintf(stderr, 
+"Error : we are linked against libpeks %s. \n\
+Install libpeks %s or make sure that\n\
+you have deleted older versions of libpeks from your system\n",
+	peks_version(), NESSUS_VERSION);
+   exit(1);
+  }
+#endif  
   addr.s_addr = htonl(INADDR_ANY);
 #ifdef USE_PTHREADS
   /* pull in library symbols - otherwise abort */
@@ -657,128 +799,146 @@ main(int argc, char * argv[])
 #endif
 #ifndef NESSUSNT
 
-  while((i=getopt(argc, argv, "CDhvc:d0123a:p:K:P:LSFQ:e"))!=-1)
+  for (;;) {
+    int option_index = 0;
+    static struct option long_options[] =
+    {
+      {"help",                 no_argument, 0, 'h'},
+      {"version",              no_argument, 0, 'v'},
+      {"delete-key",     required_argument, 0, 'K'},
+      {"change-pass-phrase",   no_argument, 0, 'C'},
+      {"make-user",      required_argument, 0, 'P'},
+      {"list-user-pwd",  required_argument, 0, 'U'},
+      {"list-keys",            no_argument, 0, 'L'},
+      {"background",           no_argument, 0, 'D'}, 
+      {"dump-cfg",             no_argument, 0, 'd'}, 
+      {"listen",         required_argument, 0, 'a'}, 
+      {"port",           required_argument, 0, 'p'}, 
+      {"config-file",    required_argument, 0, 'c'}, 
+
+      /* rpc stuff -- not active yet */
+      {"rpc-passwd",     required_argument, 0, 'Q'}, 
+      {"no-rpc-forward", required_argument, 0, 'F'}, 
+      {"rpc-server",     required_argument, 0, 'S'}, 
+      {0, 0, 0, 0}
+    };
+
+    if ((i = getopt_long 
+	 (argc, argv, "CDhvc:d0123a:p:K:P:U:LSFQ:e", 
+	  long_options, &option_index)) == EOF)
+      break;
+    else
       switch(i)
 	{
+#if 0
 	case 'e' :
 	  break;
-	  
+#endif	  
         case 'a' :
-          if(!optarg){display_help();DO_EXIT(0);}
-          if(!inet_aton(optarg, &addr)){display_help();DO_EXIT(0);}
+          if(!optarg){display_help(myself);DO_EXIT(0);}
+          if(!inet_aton(optarg, &addr)){display_help(myself);DO_EXIT(0);}
           break;
           
 	case 'p' :
-	  if(!optarg){display_help();DO_EXIT(0);}
+	  if(!optarg){display_help(myself);DO_EXIT(0);}
 	  serv_port = atoi(optarg);
 	  if((serv_port<=0)||(serv_port>=65536))
 	    {
 	      printf("Invalid port specification\n");
-	      display_help();
+	      display_help(myself);
 	      DO_EXIT(1);
 	    }
 	  break;
 
-	case 'S' :
+	case 'D' : 
+	  do_fork++;
+	  break;
+	  
+	case 'v' :
+	  print_error("nessusd (%s) %s for %s\n(C) 1998, 1999, 2000 Renaud Deraison <deraison@nessus.org>\n\n", 
+		 PROGNAME,NESSUS_VERSION, NESS_OS_NAME);
+	  DO_EXIT(0);
+	  break;
+
 #ifdef ENABLE_CRYPTO_LAYER
+	  /* rpc stuff -- not active yet */
+	case 'S' :
 	  arg_add_value (options, "ena_rpcsrv", ARG_INT, sizeof(int), (void*)1);
  	  break;
-#else
- 	  goto disp_help ;
-#endif /* ENABLE_CRYPTO_LAYER */
-
  	case 'F' :
-#ifdef ENABLE_CRYPTO_LAYER
 	  arg_add_value(options, "dis_rpcfwd", ARG_INT, sizeof(int), (void*)1);
  	  break;
-#else
- 	  goto disp_help ;
-#endif /* ENABLE_CRYPTO_LAYER */
-
  	case 'Q' :
-#ifdef ENABLE_CRYPTO_LAYER
  	  /* syntax: passwd */
 	  arg_add_value(options, "rpc_passwd", ARG_PTR, -1,  estrdup (optarg));
  	  break;
-#else
- 	  goto disp_help ;
-#endif /* ENABLE_CRYPTO_LAYER */
 
+	  /* key management and the like */
+ 	case 'U' :
+	  arg_add_value(options, "list_user_pwd", ARG_PTR, -1, optarg);
+	  break;
  	case 'P' :
-#ifdef ENABLE_CRYPTO_LAYER
- 	  /* syntax: user[,passwd] */
+ 	  /* syntax: user[,[passwd]] */
 	  if (strchr (optarg, ',') != 0) {
-	    char *usr = estrdup (optarg) ;
-	    char *pwd  = strchr (usr, ',') ;
+	    char *vsr = estrdup (optarg) ;
+	    char *pwd = strrchr (vsr, ',') ;
+	    /* resolve, if there is a password and a host/network specs */
+	    char *usr = (pwd [1] && strchr (vsr, '@'))
+	      ? tagresolve (vsr, pwd-vsr) 
+	      : vsr 
+	      ;
 	    * pwd ++ = '\0' ;
+	    if (usr == 0) {
+	      fprintf (stderr, "%s: problem with user spec \"%s\": %s.\n",
+		       myself, vsr, peks_strerr (errno));
+	      DO_EXIT(0);
+	    }
 	    arg_add_value(options, "pwd_passwd", ARG_PTR, -1, pwd);
 	    arg_add_value(options, "pwd_user",   ARG_PTR, -1, usr);
 	    break;
 	  }
 	  arg_add_value(options, "pwd_user", ARG_PTR, -1, estrdup (optarg));
  	  break;
-#else
- 	  goto disp_help ;
-#endif /* ENABLE_CRYPTO_LAYER */
- 
  	case 'K' :
-#ifdef ENABLE_CRYPTO_LAYER
  	  /* syntax: user */
 	  arg_add_value(options, "kill_userkey", ARG_PTR, -1, estrdup (optarg));
  	  break;
-#else
- 	  goto disp_help ;
-#endif /* ENABLE_CRYPTO_LAYER */
- 
  	case 'L' :
-#ifdef ENABLE_CRYPTO_LAYER
 	  arg_add_value(options, "list_udb", ARG_INT, sizeof(int), (void*)1);
 	  break;
-#else
- 	  goto disp_help ;
-#endif /* ENABLE_CRYPTO_LAYER */
-
  	case 'C' :
-#ifdef ENABLE_CRYPTO_LAYER
 	  arg_add_value(options, "chng_pph", ARG_INT, sizeof(int), (void*)1);
 	  break;
-#else
- 	  goto disp_help ;
-#endif /* ENABLE_CRYPTO_LAYER */
  
-	case 'D' : 
-	  do_fork++;
-	  break;
-	  
-	case 'v' :
-	  print_error("nessusd (%s) %s for %s\n(C) 1998-1999 Renaud Deraison <deraison@nessus.org>\n\n", 
-		 PROGNAME,NESSUS_VERSION, NESS_OS_NAME);
-	  DO_EXIT(0);
-	  break;
-#ifdef ENABLE_CRYPTO_LAYER
 	case '0' : dump_recv = 1; break;
 	case '1' : dump_send = 1; break;
 	case '2' : dump_iopt = 1; break;
- 	case '3' :
+	case '3' :
 # ifdef DEBUG
 	  single_shot = 1; 
-	  break;
 # endif
+	  break;
 #else
 	case '0' :
 	case '1' :
 	case '2' :
- 	case '3' :
+	case '3' :
+ 	case 'K' :
+ 	case 'Q' :
+ 	case 'F' :
+	case 'S' :
+ 	case 'P' :
+ 	case 'L' :
+ 	case 'C' :
 #endif
 	case 'h' :
  	case '?' : /* getopt: error */
  	case ':' : /* getopt: missing parameter */
- 	disp_help:
-	  display_help();
+	  display_help(myself);
 	  DO_EXIT(0);
 	  break;
 	case 'c' : 
-	  if(!optarg){display_help();DO_EXIT(1);}
+	  if(!optarg){display_help(myself);DO_EXIT(1);}
 	  config_file = emalloc(strlen(optarg)+1);
 	  strncpy(config_file, optarg, strlen(optarg));
 	  break;
@@ -786,21 +946,45 @@ main(int argc, char * argv[])
            printf("This is Nessus %s for %s %s\n", NESSUS_VERSION, NESS_OS_NAME, NESS_OS_VERSION);
            printf("compiled with %s\n", NESS_COMPILER);
            printf("Current setup :\n");
-           printf("\tThread manager : ");
+	  
+#ifdef ENABLE_SAVE_TESTS
+           printf("\tExperimental session-saving    : enabled\n");
+#else
+           printf("\tExperimental session-saving    : disabled\n");
+#endif
+
+#ifdef ENABLE_SAVE_KB
+	   printf("\tExperimental KB saving         : enabled\n");
+#else
+	   printf("\tExperimental KB saving         : disabled\n");
+#endif	   	   
+	  
+           printf("\tThread manager                 : ");
 #ifdef USE_PTHREADS
 	   printf("pthreads\n");
 #else
 	   printf("fork\n");
 #endif
 #ifdef ENABLE_CRYPTO_LAYER
-	   printf("\tCrypto layer : %s\n", peks_version ());
+	   printf("\tCrypto layer                   : %s\n", peks_version ());
 #endif
-	   printf("\tRunning as euid : %d\n", geteuid());
+
+	   printf("\tnasl                           : %s\n", nasl_version());
+	   printf("\tlibnessus                      : %s\n", nessuslib_version());
+
+
+	   printf("\tRunning as euid                : %d\n", geteuid());
            printf("\n\nInclude these infos in your bug reports\n");
 	   DO_EXIT(0);
 	   break;
 	}
 #endif
+  } /* end options */
+
+#if 0
+  task_loop (); /* DBUG DEBUG DEBUG */
+  exit (0);     /* DBUG DEBUG DEBUG */
+#endif
 
   if(serv_port == -1)serv_port = DEFAULT_PORT;
   if(!config_file)
@@ -820,46 +1004,6 @@ main(int argc, char * argv[])
   g_rules = arg_get_value(options, "rules");
   g_users = arg_get_value(options, "users");
 
-#if 0
-  /* this is my code to test the pty suff -- jordan */
-  arg_add_value (g_preferences, "dumpfile", ARG_STRING, 2, "-");
-  start_daemon_mode ();
-  { 
-    FILE *fp ;
-    char *cmd, **argv, buf [1000], *s ;
-
-    /* nessus_signal (SIGHUP, SIG_IGN); */
-    cmd = "/bin/cat";
-    argv = append_argv      (0, "cat");
-    fp = ptyexecvp (cmd, argv, 0);
-    destroy_argv (argv);
-    fputs ("how much wood ...\n", fp);
-    while ((s = fgets (buf, sizeof (buf), fp)) == 0 && errno == EAGAIN)
-      ;
-    while (s) {
-      printf (s);
-      s = fgets (buf, sizeof (buf), fp) ;
-    }
-    fclose (fp);
-
-    /* new game */
-
-    cmd  = "/bin/ls";
-    argv = append_argv      (0, "ls");
-    argv = append_argv (argv, "/dev");
-    fp = ptyexecvp (cmd, argv, 0);
-    destroy_argv (argv);
-    while ((s = fgets (buf, sizeof (buf), fp)) == 0 && errno == EAGAIN)
-      ;
-    while (s) {
-      printf (s);
-      s = fgets (buf, sizeof (buf), fp) ;
-    }
-    fclose (fp);
-  }
-  exit (1);
-#endif
-
 #ifdef NESSUSNT
   /*
    * Just tell to the user that nessusd is running in background
@@ -871,10 +1015,14 @@ main(int argc, char * argv[])
   {  	
    if(!fork()){
         setsid();
+	create_pid_file();
    	main_loop();
 	}
   }
-  else main_loop();
+  else {
+  	create_pid_file();
+  	main_loop();
+	}
 #endif
   DO_EXIT(0);
   return(0);
