@@ -20,11 +20,13 @@
 #define _NESSUSD_PLUGINLOAD_H
 
 struct arglist * plugins_init(struct arglist *);
+void plugin_set_socket(struct arglist *, int);
 void plugins_set_socket(struct arglist *, int);
+void plugins_free(struct arglist *);
 typedef struct pl_class_s {
     struct pl_class_s* pl_next;
     const char* extension;
-    struct pl_class_s* (*pl_init)(struct arglist*);
+    struct pl_class_s* (*pl_init)(struct arglist*, struct arglist*);
     struct arglist* (*pl_add)(char*, char*, struct arglist*, struct arglist*);
     void (*pl_launch)(struct arglist*, struct arglist *, struct arglist*,
 		      struct arglist*, struct arglist*, char *);
