@@ -22,6 +22,22 @@
 int    preferences_init(char *, struct arglist **);
 int    preferences_new(char *);
 int    preferences_process(char *, struct arglist *);
+
 int    preferences_get_host_expansion(struct arglist *);
 int    preferences_get_checks_read_timeout(struct arglist*);
+int    preferences_get_delay_between_tests(struct arglist *);
+int    preferences_log_whole_attack(struct arglist *);
+int    preferences_optimize_test(struct arglist *);
+int    preferences_ntp_show_end(struct arglist *);
+int    preferences_log_plugins_at_load(struct arglist *);
+int    preferences_plugins_timeout(struct arglist *);
+#ifdef ENABLE_SAVE_TESTS
+int    preferences_save_session(struct arglist*);
+#endif
+#ifdef ENABLE_SAVE_KB
+int    preferences_continuous_scan(struct arglist *);
+int    preferences_delay_between_scans(struct arglist *);
+int    preferences_detached_scan( struct arglist *);
+#endif
+
 #endif
