@@ -26,6 +26,7 @@
 #include "auth.h"
 #include "threads.h"
 #include "sighand.h"
+#include "utils.h"
 
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
@@ -37,6 +38,7 @@
 #include <sys/resource.h>
 #endif
 
+
 /* do not leave a zombie, hanging around if possible */
 void
 let_em_die
@@ -106,6 +108,8 @@ void (*nessus_signal(int signum, void (*handler)(int)))(int)
 
   /* Init new handler */
   sigfillset(&saNew.sa_mask);
+  sigdelset(&saNew.sa_mask, SIGALRM); /* make sleep() work */
+  
   saNew.sa_flags = 0;
 # ifdef HAVE_SIGNAL_SA_RESTORER
   saNew.sa_restorer = 0; /* not avail on Solaris - jordan */
@@ -140,12 +144,15 @@ void sighand_alarm()
   _EXIT(1);   
 }           
 
+
 void sighand_term()
 {
   log_write("received the TERM signal\n");
 #ifndef USE_AF_INET
   unlink(AF_UNIX_PATH);
 #endif
+
+  delete_pid_file();
   make_em_die (SIGTERM);
   EXIT(0);
 }
@@ -174,6 +181,7 @@ void sighand_segv()
 void sighand_kill()
 {
   log_write("received the KILL signal\n");
+  delete_pid_file();
   make_em_die (SIGTERM);
 #ifndef USE_AF_INET
   unlink(AF_UNIX_PATH);
