@@ -19,14 +19,32 @@
 #ifndef _NESSUSD_UTILS_H
 #define _NESSUSD_UTILS_H
 
+
+struct attack_atom
+{
+ char * name;
+ int soc;  /* public socket  */
+ int psoc; /* private socket */
+ struct attack_atom * next;
+};
 struct arglist * sort_plugins_by_type(struct arglist *);
 int get_max_thread_number(struct arglist *);
 int get_active_plugins_number(struct arglist *);
 void plugins_set_ntp_caps(struct arglist *, ntp_caps*);
 void send_plugin_order(struct arglist *, struct arglist *);
-int check_client_input(struct arglist *);
-void check_client_input_thread(struct arglist *);
+int check_threads_input(struct attack_atom **, int, struct arglist *);
 int is_symlink(char *);
 void check_symlink(char *);
 char * hosts_arglist_to_string(struct arglist *);
+
+struct attack_atom ** attack_atom_new();
+void attack_atom_free(struct attack_atom **);
+void attack_atom_insert(struct attack_atom **, char *, int, int);
+void attack_atom_remove(struct attack_atom **, char *);
+
+
+void create_pid_file();
+void delete_pid_file();
+
+int version_check(char *, char*);
 #endif
